using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Reflection;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Threading;

namespace Microsoft.SmallBasic.Library.Internal
{
	/// <summary>
	/// The Application class provides a Small Basic program with an application object.
	/// </summary>
	public sealed class SmallBasicApplication
	{
		private static Thread _applicationThread;

		private static Dispatcher _dispatcher;

		private static Application _application;

		internal static Queue<InvokeHelper> mainThreadActions;

		private static int _pendingOperations;

		/// <summary>
		/// Gets the dispatcher for the Small Basic application
		/// </summary>
		internal static Dispatcher Dispatcher => _dispatcher;

		internal static bool HasShutdown => Dispatcher.HasShutdownFinished;

		static SmallBasicApplication()
		{
			mainThreadActions = new Queue<InvokeHelper>();
			_pendingOperations = 0;
			AutoResetEvent autoEvent = new AutoResetEvent(initialState: false);
			_applicationThread = new Thread((ThreadStart)delegate
			{
				Application application = new Application();
				autoEvent.Set();
				_application = application;
				application.Run();
			});
			_applicationThread.SetApartmentState(ApartmentState.STA);
			_applicationThread.Start();
			autoEvent.WaitOne();
			_dispatcher = Dispatcher.FromThread(_applicationThread);
		}

		public static void BeginProgram()
		{
			AppDomain.CurrentDomain.UnhandledException += HandleException;
		}

		internal static BitmapSource CropBitmap(BitmapSource original, Primitive left, Primitive top, Primitive width, Primitive height)
		{
			if ((bool)(left < 0))
			{
				left = 0;
			}
			if ((bool)(top < 0))
			{
				top = 0;
			}
			if ((bool)(left + width > original.PixelWidth))
			{
				width = original.PixelWidth - left;
			}
			if ((bool)(top + height > original.PixelHeight))
			{
				height = original.PixelHeight - top;
			}
			return (BitmapSource)InvokeWithReturn(() => new CroppedBitmap(original, new Int32Rect(left, top, width, height)));
		}

		internal static void End()
		{
			Invoke(delegate
			{
				_application.Shutdown();
				_dispatcher.InvokeShutdown();
			});
			if (AppDomain.CurrentDomain.FriendlyName != "Debuggee")
			{
				Process.GetCurrentProcess().Kill();
			}
		}

		internal static void BeginInvoke(InvokeHelper invokeDelegate)
		{
			_pendingOperations++;
			_dispatcher.BeginInvoke(DispatcherPriority.Render, invokeDelegate);
			if (_pendingOperations > 40)
			{
				ClearDispatcherQueue();
				_pendingOperations = 0;
			}
		}

		internal static string GetEntryAssemblyPath()
		{
			return Assembly.GetEntryAssembly().Location;
		}

		internal static Uri GetResourceUri(string resourceName)
		{
			return new Uri($"pack://application:,,/SmallBasicLibrary;component/Resources/{resourceName}");
		}

		private static void HandleException(object sender, UnhandledExceptionEventArgs e)
		{
			Exception exception = e.ExceptionObject as Exception;
			if (exception != null)
			{
				Invoke(delegate
				{
					Window window = new Window
					{
						Title = "Error in Small Basic Program",
						SizeToContent = SizeToContent.WidthAndHeight,
						WindowStyle = WindowStyle.SingleBorderWindow,
						ShowInTaskbar = false,
						ResizeMode = ResizeMode.NoResize,
						Topmost = true,
						WindowStartupLocation = WindowStartupLocation.CenterScreen,
						Content = new StackPanel
						{
							Margin = new Thickness(4.0),
							Children = 
							{
								(UIElement)new DockPanel
								{
									Children = 
									{
										(UIElement)new Image
										{
											Source = new BitmapImage(new Uri("pack://application:,,/SmallBasicLibrary;component/Resources/Error.png")),
											Width = 48.0,
											Height = 48.0,
											Margin = new Thickness(8.0),
											VerticalAlignment = VerticalAlignment.Top
										},
										(UIElement)new StackPanel
										{
											Children = 
											{
												(UIElement)new TextBlock
												{
													Text = exception.Message,
													Margin = new Thickness(4.0),
													FontSize = 16.0,
													Foreground = new SolidColorBrush(Color.FromArgb(byte.MaxValue, 0, 42, 181))
												},
												(UIElement)new TextBox
												{
													Text = exception.StackTrace,
													FontFamily = new FontFamily("Consolas"),
													IsReadOnly = true,
													MaxWidth = 480.0,
													Margin = new Thickness(4.0),
													HorizontalScrollBarVisibility = ScrollBarVisibility.Auto,
													VerticalScrollBarVisibility = ScrollBarVisibility.Auto
												}
											}
										}
									}
								},
								(UIElement)new StackPanel
								{
									Orientation = Orientation.Horizontal,
									HorizontalAlignment = HorizontalAlignment.Right,
									Children = 
									{
										(UIElement)new Button
										{
											Content = "OK",
											Margin = new Thickness(4.0),
											Width = 60.0,
											IsCancel = true
										}
									}
								}
							}
						}
					};
					window.ShowDialog();
				});
			}
		}

		internal static void Invoke(InvokeHelper invokeDelegate)
		{
			_dispatcher.Invoke(DispatcherPriority.Render, invokeDelegate);
		}

		internal static object InvokeWithReturn(InvokeHelperWithReturn invokeDelegate)
		{
			return _dispatcher.Invoke(DispatcherPriority.Render, invokeDelegate);
		}

		internal static void ClearDispatcherQueue()
		{
			try
			{
				_dispatcher.Invoke(DispatcherPriority.Background, (DispatcherOperationCallback)((object param0) => null), null);
			}
			catch
			{
			}
		}

		public static void EndProgram()
		{
			if ((bool)((Primitive)(!GraphicsWindow._windowVisible) && Timer.Interval == 100000000))
			{
				Invoke(delegate
				{
					_application.Shutdown();
					_dispatcher.InvokeShutdown();
				});
				if (AppDomain.CurrentDomain.FriendlyName != "Debuggee")
				{
					Process.GetCurrentProcess().Kill();
				}
			}
		}
	}
}
