using System.Collections.Generic;

namespace Microsoft.SmallBasic.Library
{
	/// <summary>
	/// This object provides a way of storing more than one value for a given name. These values can be accessed by another index.
	/// </summary>
	[SmallBasicType]
	public sealed class Array
	{
		private static Dictionary<Primitive, Dictionary<Primitive, Primitive>> _arrayMap = new Dictionary<Primitive, Dictionary<Primitive, Primitive>>();

		/// <summary>
		/// Gets whether or not the array contains the specified index.  This is very useful when deciding if the array's index was initialized by some value or not.
		/// </summary>
		/// <param name="array">
		/// The array to check.
		/// </param>
		/// <param name="index">
		/// The index to check.
		/// </param>
		/// <returns>
		/// "True" or "False" depending on if the index was present in the specified.
		/// array.
		/// </returns>
		public static Primitive ContainsIndex(Primitive array, Primitive index)
		{
			return array.ContainsKey(index);
		}

		/// <summary>
		/// Gets whether or not the array contains the specified value.  This is very useful when deciding if the array's value was stored in some index.
		/// </summary>
		/// <param name="array">
		/// The array to check.
		/// </param>
		/// <param name="value">
		/// The value to check.
		/// </param>
		/// <returns>
		/// "True" or "False" depending on if the value was present in the specified.
		/// array.
		/// </returns>
		public static Primitive ContainsValue(Primitive array, Primitive value)
		{
			return array.ContainsValue(value);
		}

		/// <summary>
		/// Gets all the indices for the array, as another array.
		/// </summary>
		/// <param name="array">
		/// The array whose indices are requested.
		/// </param>
		/// <returns>
		/// An array filled with all the indices of the specified array.  The index of the returned array starts from 1.
		/// </returns>
		public static Primitive GetAllIndices(Primitive array)
		{
			return array.GetAllIndices();
		}

		/// <summary>
		/// Gets the number of items stored in the array.
		/// </summary>
		/// <param name="array">
		/// The array for which the count is requested.
		/// </param>
		/// <returns>
		/// The number of items in the specified array.
		/// </returns>
		public static Primitive GetItemCount(Primitive array)
		{
			return array.GetItemCount();
		}

		/// <summary>
		/// Gets whether or not a given variable is an array.
		/// </summary>
		/// <param name="array">
		/// The variable to check.
		/// </param>
		/// <returns>
		/// "True" if the specified variable is an array.  "False" otherwise.
		/// </returns>
		public static Primitive IsArray(Primitive array)
		{
			return array.IsArray;
		}

		/// <summary>
		/// Sets a value for a given array and index.
		/// </summary>
		/// <param name="arrayName">
		/// The name of the array.
		/// </param>
		/// <param name="index">
		/// Name of the index.
		/// </param>
		/// <param name="value">
		/// The value to set.
		/// </param>
		[HideFromIntellisense]
		public static void SetValue(Primitive arrayName, Primitive index, Primitive value)
		{
			if (!arrayName.IsEmpty)
			{
				if (!_arrayMap.TryGetValue(arrayName, out var value2))
				{
					value2 = new Dictionary<Primitive, Primitive>();
					_arrayMap[arrayName] = value2;
				}
				value2[index] = value;
			}
		}

		/// <summary>
		/// Gets a value for a given array and index.
		/// </summary>
		/// <param name="arrayName">
		/// The name of the array.
		/// </param>
		/// <param name="index">
		/// The name of the index.
		/// </param>
		/// <returns>
		/// The value at the specified index of the specified array.
		/// </returns>
		[HideFromIntellisense]
		public static Primitive GetValue(Primitive arrayName, Primitive index)
		{
			Dictionary<Primitive, Primitive> array = GetArray(arrayName);
			if (array == null)
			{
				return "";
			}
			if (array.TryGetValue(index, out var value))
			{
				return value;
			}
			return "";
		}

		/// <summary>
		/// Removes the array item at the specified index.
		/// </summary>
		/// <param name="arrayName">
		/// The name of the array.
		/// </param>
		/// <param name="index">
		/// The index of the item to remove.
		/// </param>
		[HideFromIntellisense]
		public static void RemoveValue(Primitive arrayName, Primitive index)
		{
			GetArray(arrayName)?.Remove(index);
		}

		internal static Dictionary<Primitive, Primitive> GetArray(Primitive arrayName)
		{
			if (_arrayMap.TryGetValue(arrayName, out var value))
			{
				return value;
			}
			return null;
		}
	}
}
