using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;

namespace Microsoft.SmallBasic.Library
{
	/// <summary>
	/// The Controls object allows you to add, move and interact with controls.
	/// </summary>
	[SmallBasicType]
	public sealed class Controls
	{
		private static Dictionary<string, Point> _positionMap = new Dictionary<string, Point>();

		private static Primitive _lastClickedButton;

		private static Primitive _lastTypedTextBox;

		/// <summary>
		/// Gets the last Button that was clicked on the Graphics Window.
		/// </summary>
		public static Primitive LastClickedButton => _lastClickedButton;

		/// <summary>
		/// Gets the last TextBox, text was typed into.
		/// </summary>
		public static Primitive LastTypedTextBox => _lastTypedTextBox;

		private static event SmallBasicCallback _buttonClicked;

		private static event SmallBasicCallback _textTyped;

		/// <summary>
		/// Raises an event when any button control is clicked.
		/// </summary>
		public static event SmallBasicCallback ButtonClicked
		{
			add
			{
				Controls._buttonClicked = null;
				_buttonClicked += value;
			}
			remove
			{
				_buttonClicked -= value;
			}
		}

		/// <summary>
		/// Raises an event when text is typed into any TextBox control.
		/// </summary>
		public static event SmallBasicCallback TextTyped
		{
			add
			{
				Controls._textTyped = null;
				_textTyped += value;
			}
			remove
			{
				_textTyped -= value;
			}
		}

		/// <summary>
		/// Adds a button to the graphics window at the specified position.
		/// </summary>
		/// <param name="caption">
		/// The caption to display in the button.
		/// </param>
		/// <param name="left">
		/// The x co-ordinate of the button.
		/// </param>
		/// <param name="top">
		/// The y co-ordinate of the button.
		/// </param>
		/// <returns>
		/// The button that was just added to the Graphics Window.
		/// </returns>
		public static Primitive AddButton(Primitive caption, Primitive left, Primitive top)
		{
			string name = Shapes.GenerateNewName("Button");
			GraphicsWindow.Invoke(delegate
			{
				Button button = new Button
				{
					Content = caption,
					Padding = new Thickness(4.0)
				};
				Canvas.SetLeft(button, left);
				Canvas.SetTop(button, top);
				button.Click += OnButtonClicked;
				GraphicsWindow.AddControl(name, button);
			});
			return name;
		}

		/// <summary>
		/// Gets the current caption of the specified button.
		/// </summary>
		/// <param name="buttonName">
		/// The Button whose caption is requested.
		/// </param>
		/// <returns>
		/// The current caption of the button.
		/// </returns>
		public static Primitive GetButtonCaption(Primitive buttonName)
		{
			if (!GraphicsWindow._objectsMap.TryGetValue(buttonName, out var value))
			{
				return "";
			}
			Button button = value as Button;
			if (button == null)
			{
				return "";
			}
			return (Primitive)GraphicsWindow.InvokeWithReturn(() => (Primitive)button.Content.ToString());
		}

		/// <summary>
		/// Sets the caption of the specified button.
		/// </summary>
		/// <param name="buttonName">
		/// The Button whose caption needs to be set.
		/// </param>
		/// <param name="caption">
		/// The new caption for the button.
		/// </param>
		public static void SetButtonCaption(Primitive buttonName, Primitive caption)
		{
			if (!GraphicsWindow._objectsMap.TryGetValue(buttonName, out var value))
			{
				return;
			}
			Button button = value as Button;
			if (button != null)
			{
				GraphicsWindow.BeginInvoke(delegate
				{
					button.Content = caption;
				});
			}
		}

		/// <summary>
		/// Adds a text input box to the graphics window at the specified position.
		/// </summary>
		/// <param name="left">
		/// The x co-ordinate of the text box.
		/// </param>
		/// <param name="top">
		/// The y co-ordinate of the text box.
		/// </param>
		/// <returns>
		/// The text box that was just added to the Graphics Window.
		/// </returns>
		public static Primitive AddTextBox(Primitive left, Primitive top)
		{
			string name = Shapes.GenerateNewName("TextBox");
			GraphicsWindow.Invoke(delegate
			{
				TextBox textBox = new TextBox
				{
					Width = 160.0,
					Padding = new Thickness(2.0)
				};
				Canvas.SetLeft(textBox, left);
				Canvas.SetTop(textBox, top);
				textBox.TextChanged += OnTextChanged;
				GraphicsWindow.AddControl(name, textBox);
			});
			return name;
		}

		/// <summary>
		/// Adds a multi-line text input box to the graphics window at the specified position.
		/// </summary>
		/// <param name="left">
		/// The x co-ordinate of the text box.
		/// </param>
		/// <param name="top">
		/// The y co-ordinate of the text box.
		/// </param>
		/// <returns>
		/// The text box that was just added to the Graphics Window.
		/// </returns>
		public static Primitive AddMultiLineTextBox(Primitive left, Primitive top)
		{
			string name = Shapes.GenerateNewName("TextBox");
			GraphicsWindow.Invoke(delegate
			{
				TextBox textBox = new TextBox
				{
					AcceptsReturn = true,
					Width = 200.0,
					Height = 80.0,
					HorizontalScrollBarVisibility = ScrollBarVisibility.Auto,
					VerticalScrollBarVisibility = ScrollBarVisibility.Auto,
					Padding = new Thickness(2.0)
				};
				Canvas.SetLeft(textBox, left);
				Canvas.SetTop(textBox, top);
				textBox.TextChanged += OnTextChanged;
				GraphicsWindow.AddControl(name, textBox);
			});
			return name;
		}

		/// <summary>
		/// Gets the current text of the specified TextBox.
		/// </summary>
		/// <param name="textBoxName">
		/// The TextBox whose text is requested.
		/// </param>
		/// <returns>
		/// The text in the TextBox
		/// </returns>
		public static Primitive GetTextBoxText(Primitive textBoxName)
		{
			if (!GraphicsWindow._objectsMap.TryGetValue(textBoxName, out var value))
			{
				return "";
			}
			TextBox textBox = value as TextBox;
			if (textBox == null)
			{
				return "";
			}
			return (Primitive)GraphicsWindow.InvokeWithReturn(() => (Primitive)textBox.Text);
		}

		/// <summary>
		/// Sets the text of the specified TextBox.
		/// </summary>
		/// <param name="textBoxName">
		/// The TextBox whose text needs to be set.
		/// </param>
		/// <param name="text">
		/// The new text for the TextBox.
		/// </param>
		public static void SetTextBoxText(Primitive textBoxName, Primitive text)
		{
			if (!GraphicsWindow._objectsMap.TryGetValue(textBoxName, out var value))
			{
				return;
			}
			TextBox textBox = value as TextBox;
			if (textBox != null)
			{
				GraphicsWindow.BeginInvoke(delegate
				{
					textBox.Text = text;
				});
			}
		}

		/// <summary>
		/// Removes a control from the Graphics Window.
		/// </summary>
		/// <param name="controlName">
		/// The name of the control that needs to be removed.
		/// </param>
		public static void Remove(Primitive controlName)
		{
			if (GraphicsWindow._objectsMap.TryGetValue(controlName, out var value))
			{
				Button button = value as Button;
				if (button != null)
				{
					button.Click -= OnButtonClicked;
				}
				GraphicsWindow.RemoveShape(controlName);
			}
		}

		/// <summary>
		/// Moves the control with the specified name to a new position.
		/// </summary>
		/// <param name="control">
		/// The name of the control to move.
		/// </param>
		/// <param name="x">
		/// The x co-ordinate of the new position.
		/// </param>
		/// <param name="y">
		/// The y co-ordinate of the new position.
		/// </param>
		public static void Move(Primitive control, Primitive x, Primitive y)
		{
			if (GraphicsWindow._objectsMap.TryGetValue(control, out var obj))
			{
				_positionMap[control] = new Point(x, y);
				GraphicsWindow.BeginInvoke(delegate
				{
					obj.BeginAnimation(Canvas.LeftProperty, null);
					obj.BeginAnimation(Canvas.TopProperty, null);
					Canvas.SetLeft(obj, x);
					Canvas.SetTop(obj, y);
				});
			}
		}

		/// <summary>
		/// Sets the size of the control.
		/// </summary>
		/// <param name="control">
		/// The name of the control to be resized.
		/// </param>
		/// <param name="width">
		/// The width of the control.
		/// </param>
		/// <param name="height">
		/// The height of the control.
		/// </param>
		public static void SetSize(Primitive control, Primitive width, Primitive height)
		{
			if (!GraphicsWindow._objectsMap.TryGetValue(control, out var value))
			{
				return;
			}
			FrameworkElement element = value as FrameworkElement;
			if (element != null)
			{
				GraphicsWindow.BeginInvoke(delegate
				{
					element.Width = width;
					element.Height = height;
				});
			}
		}

		/// <summary>
		/// Hides an already added control.
		/// </summary>
		/// <param name="controlName">
		/// The name of the control.
		/// </param>
		public static void HideControl(Primitive controlName)
		{
			if (GraphicsWindow._objectsMap.TryGetValue(controlName, out var obj))
			{
				GraphicsWindow.Invoke(delegate
				{
					obj.Visibility = Visibility.Collapsed;
				});
			}
		}

		/// <summary>
		/// Shows a previously hidden control.
		/// </summary>
		/// <param name="controlName">
		/// The name of the control.
		/// </param>
		public static void ShowControl(Primitive controlName)
		{
			if (GraphicsWindow._objectsMap.TryGetValue(controlName, out var obj))
			{
				GraphicsWindow.Invoke(delegate
				{
					obj.Visibility = Visibility.Visible;
				});
			}
		}

		private static void OnButtonClicked(object sender, RoutedEventArgs e)
		{
			Button button = sender as Button;
			string name = button.Name;
			if (GraphicsWindow._objectsMap.ContainsKey(name))
			{
				_lastClickedButton = name;
				Controls._buttonClicked?.Invoke();
			}
		}

		private static void OnTextChanged(object sender, EventArgs e)
		{
			TextBox textBox = sender as TextBox;
			string name = textBox.Name;
			if (GraphicsWindow._objectsMap.ContainsKey(name))
			{
				_lastTypedTextBox = name;
				Controls._textTyped?.Invoke();
			}
		}
	}
}
