using System;
using System.Collections.Generic;
using System.IO;
using Microsoft.SmallBasic.Library.Internal;

namespace Microsoft.SmallBasic.Library
{
	/// <summary>
	/// The File object provides methods to access, read and write information from and to a file on disk.  Using this object, it is possible to save and open settings across multiple sessions of your program.
	/// </summary>
	[SmallBasicType]
	public sealed class File
	{
		/// <summary>
		/// <para>
		/// Gets or sets the last encountered file operation based error message.  This property is useful for finding out when some method fails to execute.
		/// </para>
		/// </summary>
		public static Primitive LastError
		{
			get;
			set;
		}

		/// <summary>
		/// <para>
		/// Opens a file and reads the entire file's contents.  This method will be fast for small files that are less than an MB in size, but will start to slow down and will be noticeable for files greater than 10MB.
		/// </para>
		/// </summary>
		/// <param name="filePath">
		/// <para>
		/// The full path of the file to read.  An example of a full path will be c:\temp\settings.data.
		/// </para>
		/// </param>
		/// <returns>
		/// The entire contents of the file.
		/// </returns>
		public static Primitive ReadContents(Primitive filePath)
		{
			LastError = "";
			string path = Environment.ExpandEnvironmentVariables(filePath);
			try
			{
				if (!System.IO.File.Exists(path))
				{
					return "";
				}
				using (StreamReader streamReader = new StreamReader(path))
				{
					string value = streamReader.ReadToEnd();
					return value;
				}
			}
			catch (Exception ex)
			{
				LastError = ex.Message;
			}
			return "";
		}

		/// <summary>
		/// <para>
		/// Opens a file and writes the specified contents into it, replacing the original contents with the new content.
		/// </para>
		/// </summary>
		/// <param name="filePath">
		/// <para>
		/// The full path of the file to write to.  An example of a full path will be c:\temp\settings.data.
		/// </para>
		/// </param>
		/// <param name="contents">
		/// The contents to write into the specified file.
		/// </param>
		/// <returns>
		/// <para>
		/// If the operation was successful, this will return "SUCCESS".  Otherwise, it will return "FAILED".
		/// </para>
		/// </returns>
		public static Primitive WriteContents(Primitive filePath, Primitive contents)
		{
			LastError = "";
			string path = Environment.ExpandEnvironmentVariables(filePath);
			try
			{
				using (StreamWriter streamWriter = new StreamWriter(path))
				{
					streamWriter.Write((string)contents);
					return "SUCCESS";
				}
			}
			catch (Exception ex)
			{
				LastError = ex.Message;
			}
			return "FAILED";
		}

		/// <summary>
		/// Opens the specified file and reads the contents at the specified line number.
		/// </summary>
		/// <param name="filePath">
		/// <para>
		/// The full path of the file to read from.  An example of a full path will be c:\temp\settings.data.
		/// </para>
		/// </param>
		/// <param name="lineNumber">
		/// The line number of the text to be read.
		/// </param>
		/// <returns>
		/// The text at the specified line of the specified file.
		/// </returns>
		public static Primitive ReadLine(Primitive filePath, Primitive lineNumber)
		{
			LastError = "";
			string path = Environment.ExpandEnvironmentVariables(filePath);
			if ((bool)(lineNumber < 0))
			{
				return "";
			}
			try
			{
				if (!System.IO.File.Exists(path))
				{
					return "";
				}
				using (StreamReader streamReader = new StreamReader(path))
				{
					for (int i = 0; i < (int)lineNumber - 1; i++)
					{
						if (streamReader.ReadLine() == null)
						{
							return "";
						}
					}
					return streamReader.ReadLine();
				}
			}
			catch (Exception ex)
			{
				LastError = ex.Message;
			}
			return "";
		}

		/// <summary>
		/// <para>
		/// Opens the specified file and write the contents at the specified line number.
		/// </para>
		/// <para>
		/// This operation will overwrite any existing content at the specified line.
		/// </para>
		/// </summary>
		/// <param name="filePath">
		/// <para>
		/// The full path of the file to read from.  An example of a full path will be c:\temp\settings.data.
		/// </para>
		/// </param>
		/// <param name="lineNumber">
		/// The line number of the text to write.
		/// </param>
		/// <param name="contents">
		/// <para>
		/// The contents to write at the specified line of the specified file.
		/// </para>
		/// </param>
		/// <returns>
		/// <para>
		/// If the operation was successful, this will return "SUCCESS".  Otherwise, it will return "FAILED".
		/// </para>
		/// </returns>
		public static Primitive WriteLine(Primitive filePath, Primitive lineNumber, Primitive contents)
		{
			LastError = "";
			string path = Environment.ExpandEnvironmentVariables(filePath);
			string tempFileName = Path.GetTempFileName();
			try
			{
				if (!System.IO.File.Exists(path))
				{
					using (StreamWriter streamWriter = new StreamWriter(path))
					{
						streamWriter.WriteLine((string)contents);
					}
					return "SUCCESS";
				}
				using (StreamWriter streamWriter2 = new StreamWriter(tempFileName))
				{
					using (StreamReader streamReader = new StreamReader(path))
					{
						int num = 1;
						while (true)
						{
							string text = streamReader.ReadLine();
							if (text == null)
							{
								break;
							}
							if ((bool)(num == lineNumber))
							{
								streamWriter2.WriteLine((string)contents);
							}
							else
							{
								streamWriter2.WriteLine(text);
							}
							num++;
						}
						if ((bool)(num <= lineNumber))
						{
							streamWriter2.WriteLine((string)contents);
						}
					}
				}
				System.IO.File.Copy(tempFileName, filePath, overwrite: true);
				System.IO.File.Delete(tempFileName);
				return "SUCCESS";
			}
			catch (Exception ex)
			{
				LastError = ex.Message;
			}
			return "FAILED";
		}

		/// <summary>
		/// <para>
		/// Opens the specified file and inserts the contents at the specified line number.
		/// </para>
		/// <para>
		/// This operation will not overwrite any existing content at the specified line.
		/// </para>
		/// </summary>
		/// <param name="filePath">
		/// <para>
		/// The full path of the file to read from.  An example of a full path will be c:\temp\settings.data.
		/// </para>
		/// </param>
		/// <param name="lineNumber">
		/// The line number of the text to insert.
		/// </param>
		/// <param name="contents">
		/// The contents to insert into the file.
		/// </param>
		/// <returns>
		/// <para>
		/// If the operation was successful, this will return "SUCCESS".  Otherwise, it will return "FAILED".
		/// </para>
		/// </returns>
		public static Primitive InsertLine(Primitive filePath, Primitive lineNumber, Primitive contents)
		{
			LastError = "";
			string path = Environment.ExpandEnvironmentVariables(filePath);
			string tempFileName = Path.GetTempFileName();
			try
			{
				if (!System.IO.File.Exists(path))
				{
					using (StreamWriter streamWriter = new StreamWriter(path))
					{
						streamWriter.WriteLine((string)contents);
					}
					return "SUCCESS";
				}
				using (StreamWriter streamWriter2 = new StreamWriter(tempFileName))
				{
					using (StreamReader streamReader = new StreamReader(path))
					{
						int num = 1;
						while (true)
						{
							string text = streamReader.ReadLine();
							if (text == null)
							{
								break;
							}
							if ((bool)(num == lineNumber))
							{
								streamWriter2.WriteLine((string)contents);
							}
							streamWriter2.WriteLine(text);
							num++;
						}
						if ((bool)(num <= lineNumber))
						{
							streamWriter2.WriteLine((string)contents);
						}
					}
				}
				System.IO.File.Copy(tempFileName, filePath, overwrite: true);
				System.IO.File.Delete(tempFileName);
				return "SUCCESS";
			}
			catch (Exception ex)
			{
				LastError = ex.Message;
			}
			return "FAILED";
		}

		/// <summary>
		/// <para>
		/// Opens the specified file and appends the contents to the end of the file.
		/// </para>
		/// </summary>
		/// <param name="filePath">
		/// <para>
		/// The full path of the file to read from.  An example of a full path will be c:\temp\settings.data.
		/// </para>
		/// </param>
		/// <param name="contents">
		/// The contents to append to the end of the file.
		/// </param>
		/// <returns>
		/// <para>
		/// If the operation was successful, this will return "SUCCESS".  Otherwise, it will return "FAILED".
		/// </para>
		/// </returns>
		public static Primitive AppendContents(Primitive filePath, Primitive contents)
		{
			LastError = "";
			Environment.ExpandEnvironmentVariables(filePath);
			try
			{
				using (StreamWriter streamWriter = new StreamWriter(filePath, append: true))
				{
					streamWriter.WriteLine((string)contents);
				}
				return "SUCCESS";
			}
			catch (Exception ex)
			{
				LastError = ex.Message;
			}
			return "FAILED";
		}

		/// <summary>
		/// <para>
		/// Copies the specified source file to the destination file path.  If the destination points to a location that doesn't exist, the method will attempt to create it automatically.
		/// </para>
		/// <para>
		/// Existing files will be overwritten.  It is always best to check if the destination file exists if you don't want to overwrite existing files.
		/// </para>
		/// </summary>
		/// <param name="sourceFilePath">
		/// <para>
		/// The full path of the file that needs to be copied.  An example of a full path will be c:\temp\settings.data.
		/// </para>
		/// </param>
		/// <param name="destinationFilePath">
		/// The destination location or the file path.  
		/// </param>
		/// <returns>
		/// If the operation was successful, this will return "SUCCESS".  Otherwise, it will return "FAILED".
		/// </returns>
		public static Primitive CopyFile(Primitive sourceFilePath, Primitive destinationFilePath)
		{
			LastError = "";
			string text = Environment.ExpandEnvironmentVariables(sourceFilePath);
			string text2 = Environment.ExpandEnvironmentVariables(destinationFilePath);
			if (!System.IO.File.Exists(text))
			{
				LastError = "Source file doesn't exist.";
				return "FAILED";
			}
			if (Directory.Exists(text2) || text2[text2.Length - 1] == '\\')
			{
				text2 = Path.Combine(text2, Path.GetFileName(text));
			}
			try
			{
				string directoryName = Path.GetDirectoryName(text2);
				if (!Directory.Exists(directoryName))
				{
					Directory.CreateDirectory(directoryName);
				}
				System.IO.File.Copy(text, text2, overwrite: true);
				return "SUCCESS";
			}
			catch (Exception ex)
			{
				LastError = ex.Message;
			}
			return "FAILED";
		}

		/// <summary>
		/// Deletes the specified file.
		/// </summary>
		/// <param name="filePath">
		/// <para>
		/// The destination location or the file path.  An example of a full path will be
		/// c:\temp\settings.data.
		/// </para>
		/// </param>
		/// <returns>
		/// If the operation was successful, this will return "SUCCESS".  Otherwise, it will return "FAILED".
		/// </returns>
		public static Primitive DeleteFile(Primitive filePath)
		{
			LastError = "";
			string path = Environment.ExpandEnvironmentVariables(filePath);
			try
			{
				System.IO.File.Delete(path);
				return "SUCCESS";
			}
			catch (Exception ex)
			{
				LastError = ex.Message;
			}
			return "FAILED";
		}

		/// <summary>
		/// Creates the specified directory.
		/// </summary>
		/// <param name="directoryPath">
		/// The full path of the directory to be created.
		/// </param>
		/// <returns>
		/// If the operation was successful, this will return "SUCCESS".  Otherwise, it will return "FAILED".
		/// </returns>
		public static Primitive CreateDirectory(Primitive directoryPath)
		{
			LastError = "";
			Environment.ExpandEnvironmentVariables(directoryPath);
			try
			{
				Directory.CreateDirectory(directoryPath);
				return "SUCCESS";
			}
			catch (Exception ex)
			{
				LastError = ex.Message;
			}
			return "FAILED";
		}

		/// <summary>
		/// Deletes the specified directory.
		/// </summary>
		/// <param name="directoryPath">
		/// The full path of the directory to be deleted.
		/// </param>
		/// <returns>
		/// If the operation was successful, this will return "SUCCESS".  Otherwise, it will return "FAILED".
		/// </returns>
		public static Primitive DeleteDirectory(Primitive directoryPath)
		{
			LastError = "";
			Environment.ExpandEnvironmentVariables(directoryPath);
			try
			{
				Directory.Delete(directoryPath, recursive: true);
				return "SUCCESS";
			}
			catch (Exception ex)
			{
				LastError = ex.Message;
			}
			return "FAILED";
		}

		/// <summary>
		/// Gets the path of all the files in the specified directory path.
		/// </summary>
		/// <param name="directoryPath">
		/// The directory to look for files.
		/// </param>
		/// <returns>
		/// If the operation was successful, this will return the files as an array.  Otherwise, it will return "FAILED".
		/// </returns>
		public static Primitive GetFiles(Primitive directoryPath)
		{
			LastError = "";
			Environment.ExpandEnvironmentVariables(directoryPath);
			try
			{
				if (Directory.Exists(directoryPath))
				{
					Dictionary<Primitive, Primitive> dictionary = new Dictionary<Primitive, Primitive>();
					int num = 1;
					string[] files = Directory.GetFiles(directoryPath);
					foreach (string value in files)
					{
						dictionary[num] = value;
						num++;
					}
					return Primitive.ConvertFromMap(dictionary);
				}
				LastError = "Directory Path does not exist.";
				return "FAILED";
			}
			catch (Exception ex)
			{
				LastError = ex.Message;
				return "FAILED";
			}
		}

		/// <summary>
		/// Gets the path of all the directories in the specified directory path.
		/// </summary>
		/// <param name="directoryPath">
		/// The directory to look for subdirectories.
		/// </param>
		/// <returns>
		/// If the operation was successful, this will return the list of directories as an array.  Otherwise, it will return "FAILED".
		/// </returns>
		public static Primitive GetDirectories(Primitive directoryPath)
		{
			LastError = "";
			Environment.ExpandEnvironmentVariables(directoryPath);
			try
			{
				if (Directory.Exists(directoryPath))
				{
					Dictionary<Primitive, Primitive> dictionary = new Dictionary<Primitive, Primitive>();
					int num = 1;
					string[] directories = Directory.GetDirectories(directoryPath);
					foreach (string value in directories)
					{
						dictionary[num] = value;
						num++;
					}
					return Primitive.ConvertFromMap(dictionary);
				}
				LastError = "Directory Path does not exist.";
				return "FAILED";
			}
			catch (Exception ex)
			{
				LastError = ex.Message;
				return "FAILED";
			}
		}

		/// <summary>
		/// <para>
		/// Creates a new temporary file in a temporary directory and returns the 
		/// full file path.
		/// </para>
		/// </summary>
		/// <returns>
		/// The full file path of the temporary file.
		/// </returns>
		public static Primitive GetTemporaryFilePath()
		{
			return Path.GetTempFileName();
		}

		/// <summary>
		/// <para>
		/// Gets the full path of the settings file for this program.  The settings file name is based on the program's name and is present in the same location as the program.
		/// </para>
		/// </summary>
		/// <returns>
		/// The full path of the settings file specific for this program.
		/// </returns>
		public static Primitive GetSettingsFilePath()
		{
			string entryAssemblyPath = SmallBasicApplication.GetEntryAssemblyPath();
			return Path.ChangeExtension(entryAssemblyPath, ".settings");
		}
	}
}
