using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Globalization;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Interop;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using Microsoft.SmallBasic.Library.Internal;

namespace Microsoft.SmallBasic.Library
{
	/// <summary>
	/// The GraphicsWindow provides graphics related input and output functionality.  For example, using this class, it is possible to draw and fill circles and rectangles.
	/// </summary>
	[SmallBasicType]
	public sealed class GraphicsWindow
	{
		private class VisualContainer : FrameworkElement
		{
			private Drawing _drawing;

			public VisualContainer(Drawing drawing)
			{
				_drawing = drawing;
			}

			protected override void OnRender(DrawingContext drawingContext)
			{
				drawingContext.DrawDrawing(_drawing);
			}
		}

		private const string _defaultFontName = "Tahoma";

		private const int _step = 20;

		internal static bool _windowVisible = false;

		internal static bool _windowCreated = false;

		private static Window _window;

		internal static System.Windows.Media.Pen _pen;

		internal static SolidColorBrush _fillBrush = System.Windows.Media.Brushes.SlateBlue;

		internal static System.Windows.Media.FontFamily _fontFamily;

		internal static double _fontSize = 12.0;

		internal static System.Windows.FontStyle _fontStyle = FontStyles.Normal;

		internal static FontWeight _fontWeight = FontWeights.Bold;

		private static double _mouseX;

		private static double _mouseY;

		private static Key _lastKey;

		private static string _lastText;

		private static Random _random;

		private static SolidColorBrush _backgroundBrush;

		internal static Dictionary<string, UIElement> _objectsMap = new Dictionary<string, UIElement>();

		private static Dictionary<string, ScaleTransform> _scaleTransformMap = new Dictionary<string, ScaleTransform>();

		private static Canvas _mainCanvas;

		private static DrawingGroup _mainDrawing;

		private static VisualContainer _visualContainer;

		private static RenderTargetBitmap _renderBitmap;

		private static System.Windows.Controls.Image _bitmapContainer;

		private static bool _isHidden;

		private static object _syncLock = new object();

		/// <summary>
		/// Gets or sets the Background color of the Graphics Window.
		/// </summary>
		public static Primitive BackgroundColor
		{
			get
			{
				VerifyAccess();
				return (string)InvokeWithReturn(() => GetStringFromColor(_backgroundBrush.Color));
			}
			set
			{
				VerifyAccess();
				BeginInvoke(delegate
				{
					_backgroundBrush = new SolidColorBrush(GetColorFromString(value));
					_window.Background = _backgroundBrush;
				});
			}
		}

		/// <summary>
		/// Gets or sets the brush color to be used to fill shapes drawn on the Graphics Window.
		/// </summary>
		public static Primitive BrushColor
		{
			get
			{
				VerifyAccess();
				return (string)InvokeWithReturn(() => GetStringFromColor(_fillBrush.Color));
			}
			set
			{
				VerifyAccess();
				BeginInvoke(delegate
				{
					_fillBrush = new SolidColorBrush(GetColorFromString(value));
					_fillBrush.Freeze();
				});
			}
		}

		/// <summary>
		/// Specifies whether or not the Graphics Window can be resized by the user.
		/// </summary>
		public static Primitive CanResize
		{
			get
			{
				VerifyAccess();
				return (bool)InvokeWithReturn(() => _window.ResizeMode == ResizeMode.CanResize);
			}
			set
			{
				VerifyAccess();
				BeginInvoke(delegate
				{
					if ((bool)value)
					{
						_window.ResizeMode = ResizeMode.CanResize;
					}
					else
					{
						_window.ResizeMode = ResizeMode.NoResize;
					}
				});
			}
		}

		/// <summary>
		/// Gets or sets the width of the pen used to draw shapes on the Graphics Window.
		/// </summary>
		public static Primitive PenWidth
		{
			get
			{
				VerifyAccess();
				return (double)InvokeWithReturn(() => (_pen != null) ? ((object)_pen.Thickness) : ((object)2.0));
			}
			set
			{
				VerifyAccess();
				BeginInvoke(delegate
				{
					_pen = new System.Windows.Media.Pen(_pen.Brush, value);
					_pen.Freeze();
				});
			}
		}

		/// <summary>
		/// Gets or sets the color of the pen used to draw shapes on the Graphics Window.
		/// </summary>
		public static Primitive PenColor
		{
			get
			{
				VerifyAccess();
				return (string)InvokeWithReturn(() => (_pen != null) ? GetStringFromColor(((SolidColorBrush)_pen.Brush).Color) : "Black");
			}
			set
			{
				VerifyAccess();
				BeginInvoke(delegate
				{
					System.Windows.Media.Color colorFromString = GetColorFromString(value);
					_pen = new System.Windows.Media.Pen(new SolidColorBrush(colorFromString), _pen.Thickness);
					_pen.Freeze();
				});
			}
		}

		/// <summary>
		/// Gets or sets the Font Name to be used when drawing text on the Graphics Window.
		/// </summary>
		public static Primitive FontName
		{
			get
			{
				return (string)InvokeWithReturn(() => (_fontFamily != null) ? _fontFamily.Source : "Tahoma");
			}
			set
			{
				BeginInvoke(delegate
				{
					_fontFamily = new System.Windows.Media.FontFamily(value);
				});
			}
		}

		/// <summary>
		/// Gets or sets the Font Size to be used when drawing text on the Graphics Window.
		/// </summary>
		public static Primitive FontSize
		{
			get
			{
				return new Primitive(_fontSize);
			}
			set
			{
				BeginInvoke(delegate
				{
					_fontSize = value;
				});
			}
		}

		/// <summary>
		/// Gets or sets whether or not the font to be used when drawing text on the Graphics Window, is bold.
		/// </summary>
		public static Primitive FontBold
		{
			get
			{
				return _fontWeight == FontWeights.Bold;
			}
			set
			{
				BeginInvoke(delegate
				{
					if ((bool)value)
					{
						_fontWeight = FontWeights.Bold;
					}
					else
					{
						_fontWeight = FontWeights.Normal;
					}
				});
			}
		}

		/// <summary>
		/// Gets or sets whether or not the font to be used when drawing text on the Graphics Window, is italic.
		/// </summary>
		public static Primitive FontItalic
		{
			get
			{
				return _fontStyle == FontStyles.Italic;
			}
			set
			{
				BeginInvoke(delegate
				{
					if ((bool)value)
					{
						_fontStyle = FontStyles.Italic;
					}
					else
					{
						_fontStyle = FontStyles.Normal;
					}
				});
			}
		}

		/// <summary>
		/// Gets or sets the title for the graphics window.
		/// </summary>
		public static Primitive Title
		{
			get
			{
				VerifyAccess();
				return (string)InvokeWithReturn(() => _window.Title);
			}
			set
			{
				VerifyAccess();
				BeginInvoke(delegate
				{
					_window.Title = value;
				});
			}
		}

		/// <summary>
		/// Gets or sets the Height of the graphics window.
		/// </summary>
		public static Primitive Height
		{
			get
			{
				VerifyAccess();
				return (double)InvokeWithReturn(() => _mainCanvas.ActualHeight);
			}
			set
			{
				VerifyAccess();
				BeginInvoke(delegate
				{
					_window.Height = value + (_window.ActualHeight - _mainCanvas.ActualHeight);
				});
			}
		}

		/// <summary>
		/// Gets or sets the Width of the graphics window.
		/// </summary>
		public static Primitive Width
		{
			get
			{
				VerifyAccess();
				return (double)InvokeWithReturn(() => _mainCanvas.ActualWidth);
			}
			set
			{
				VerifyAccess();
				BeginInvoke(delegate
				{
					_window.Width = value + (_window.ActualWidth - _mainCanvas.ActualWidth);
				});
			}
		}

		/// <summary>
		/// Gets or sets the Left Position of the graphics window.
		/// </summary>
		public static Primitive Left
		{
			get
			{
				VerifyAccess();
				return (double)InvokeWithReturn(() => _window.Left);
			}
			set
			{
				VerifyAccess();
				BeginInvoke(delegate
				{
					_window.Left = value;
				});
			}
		}

		/// <summary>
		/// Gets or sets the Top Position of the graphics window.
		/// </summary>
		public static Primitive Top
		{
			get
			{
				VerifyAccess();
				return (double)InvokeWithReturn(() => _window.Top);
			}
			set
			{
				VerifyAccess();
				BeginInvoke(delegate
				{
					_window.Top = value;
				});
			}
		}

		/// <summary>
		/// Gets the last key that was pressed or released.
		/// </summary>
		public static Primitive LastKey => _lastKey.ToString();

		/// <summary>
		/// Gets the last text that was entered on the Graphics Window.
		/// </summary>
		public static Primitive LastText => _lastText.ToString();

		/// <summary>
		/// Gets the x-position of the mouse relative to the Graphics Window.
		/// </summary>
		public static Primitive MouseX => _mouseX;

		/// <summary>
		/// Gets the y-position of the mouse relative to the Graphics Window.
		/// </summary>
		public static Primitive MouseY => _mouseY;

		private static event SmallBasicCallback _mouseDown;

		private static event SmallBasicCallback _mouseUp;

		private static event SmallBasicCallback _mouseMove;

		private static event SmallBasicCallback _keyDown;

		private static event SmallBasicCallback _keyUp;

		private static event SmallBasicCallback _textInput;

		/// <summary>
		/// Raises an event when a key is pressed down on the keyboard.
		/// </summary>
		public static event SmallBasicCallback KeyDown
		{
			add
			{
				VerifyAccess();
				GraphicsWindow._keyDown = null;
				_keyDown += value;
			}
			remove
			{
				_keyDown -= value;
			}
		}

		/// <summary>
		/// Raises an event when a key is released on the keyboard.
		/// </summary>
		public static event SmallBasicCallback KeyUp
		{
			add
			{
				VerifyAccess();
				GraphicsWindow._keyUp = null;
				_keyUp += value;
			}
			remove
			{
				_keyUp -= value;
			}
		}

		/// <summary>
		/// Raises an event when the mouse button is clicked down.
		/// </summary>
		public static event SmallBasicCallback MouseDown
		{
			add
			{
				VerifyAccess();
				GraphicsWindow._mouseDown = null;
				_mouseDown += value;
			}
			remove
			{
				_mouseDown -= value;
			}
		}

		/// <summary>
		/// Raises an event when the mouse button is released.
		/// </summary>
		public static event SmallBasicCallback MouseUp
		{
			add
			{
				VerifyAccess();
				GraphicsWindow._mouseUp = null;
				_mouseUp += value;
			}
			remove
			{
				_mouseUp -= value;
			}
		}

		/// <summary>
		/// Raises an event when the mouse is moved around.
		/// </summary>
		public static event SmallBasicCallback MouseMove
		{
			add
			{
				VerifyAccess();
				GraphicsWindow._mouseMove = null;
				_mouseMove += value;
			}
			remove
			{
				_mouseMove -= value;
			}
		}

		/// <summary>
		/// Raises an event when text is entered on the GraphicsWindow.
		/// </summary>
		public static event SmallBasicCallback TextInput
		{
			add
			{
				VerifyAccess();
				GraphicsWindow._textInput = null;
				_textInput += value;
			}
			remove
			{
				_textInput -= value;
			}
		}

		/// <summary>
		/// Shows the Graphics window to enable interactions with it.
		/// </summary>
		public static void Show()
		{
			if (!_windowCreated)
			{
				CreateWindow();
			}
			_isHidden = false;
			if (!_windowVisible)
			{
				Invoke(delegate
				{
					_window.Show();
				});
				_windowVisible = true;
			}
		}

		/// <summary>
		/// Hides the Graphics window.
		/// </summary>
		public static void Hide()
		{
			_isHidden = true;
			if (_windowVisible)
			{
				Invoke(delegate
				{
					_window.Hide();
				});
				_windowVisible = false;
			}
		}

		/// <summary>
		/// Draws a rectangle on the screen using the selected Pen.
		/// </summary>
		/// <param name="x">
		/// The x co-ordinate of the rectangle.
		/// </param>
		/// <param name="y">
		/// The y co-ordinate of the rectangle.
		/// </param>
		/// <param name="width">
		/// The width of the rectangle.
		/// </param>
		/// <param name="height">
		/// The height of the rectangle.
		/// </param>
		public static void DrawRectangle(Primitive x, Primitive y, Primitive width, Primitive height)
		{
			VerifyAccess();
			BeginInvoke(delegate
			{
				DrawingContext drawingContext = _mainDrawing.Append();
				drawingContext.DrawRectangle(null, _pen, new Rect((int)x, (int)y, width, height));
				drawingContext.Close();
				AddRasterizeOperationToQueue();
			});
		}

		/// <summary>
		/// Fills a rectangle on the screen using the selected Brush.
		/// </summary>
		/// <param name="x">
		/// The x co-ordinate of the rectangle.
		/// </param>
		/// <param name="y">
		/// The y co-ordinate of the rectangle.
		/// </param>
		/// <param name="width">
		/// The width of the rectangle.
		/// </param>
		/// <param name="height">
		/// The height of the rectangle.
		/// </param>
		public static void FillRectangle(Primitive x, Primitive y, Primitive width, Primitive height)
		{
			VerifyAccess();
			BeginInvoke(delegate
			{
				DrawingContext drawingContext = _mainDrawing.Append();
				drawingContext.DrawRectangle(_fillBrush, null, new Rect((int)x, (int)y, width, height));
				drawingContext.Close();
				AddRasterizeOperationToQueue();
			});
		}

		/// <summary>
		/// Draws an ellipse on the screen using the selected Pen.
		/// </summary>
		/// <param name="x">
		/// The x co-ordinate of the ellipse.
		/// </param>
		/// <param name="y">
		/// The y co-ordinate of the ellipse.
		/// </param>
		/// <param name="width">
		/// The width of the ellipse.
		/// </param>
		/// <param name="height">
		/// The height of the ellipse.
		/// </param>
		public static void DrawEllipse(Primitive x, Primitive y, Primitive width, Primitive height)
		{
			VerifyAccess();
			BeginInvoke(delegate
			{
				DrawingContext drawingContext = _mainDrawing.Append();
				drawingContext.DrawEllipse(null, _pen, new System.Windows.Point((int)x + width / 2, (int)y + height / 2), width / 2, height / 2);
				drawingContext.Close();
				AddRasterizeOperationToQueue();
			});
		}

		/// <summary>
		/// Fills an ellipse on the screen using the selected Brush.
		/// </summary>
		/// <param name="x">
		/// The x co-ordinate of the ellipse.
		/// </param>
		/// <param name="y">
		/// The y co-ordinate of the ellipse.
		/// </param>
		/// <param name="width">
		/// The width of the ellipse.
		/// </param>
		/// <param name="height">
		/// The height of the ellipse.
		/// </param>
		public static void FillEllipse(Primitive x, Primitive y, Primitive width, Primitive height)
		{
			VerifyAccess();
			BeginInvoke(delegate
			{
				DrawingContext drawingContext = _mainDrawing.Append();
				drawingContext.DrawEllipse(_fillBrush, null, new System.Windows.Point((int)x + width / 2, (int)y + height / 2), width / 2, height / 2);
				drawingContext.Close();
				AddRasterizeOperationToQueue();
			});
		}

		/// <summary>
		/// Draws a triangle on the screen using the selected pen.
		/// </summary>
		/// <param name="x1">
		/// The x co-ordinate of the first point.
		/// </param>
		/// <param name="y1">
		/// The y co-ordinate of the first point.
		/// </param>
		/// <param name="x2">
		/// The x co-ordinate of the second point.
		/// </param>
		/// <param name="y2">
		/// The y co-ordinate of the second point.
		/// </param>
		/// <param name="x3">
		/// The x co-ordinate of the third point.
		/// </param>
		/// <param name="y3">
		/// The y co-ordinate of the third point.
		/// </param>
		public static void DrawTriangle(Primitive x1, Primitive y1, Primitive x2, Primitive y2, Primitive x3, Primitive y3)
		{
			VerifyAccess();
			BeginInvoke(delegate
			{
				DrawingContext drawingContext = _mainDrawing.Append();
				PathFigure value = new PathFigure
				{
					StartPoint = new System.Windows.Point(x1, y1),
					IsClosed = true,
					Segments = 
					{
						(PathSegment)new LineSegment(new System.Windows.Point(x2, y2), isStroked: true),
						(PathSegment)new LineSegment(new System.Windows.Point(x3, y3), isStroked: true)
					}
				};
				PathFigureCollection figures = new PathFigureCollection
				{
					value
				};
				PathGeometry pathGeometry = new PathGeometry();
				pathGeometry.Figures = figures;
				pathGeometry.Freeze();
				drawingContext.DrawGeometry(null, _pen, pathGeometry);
				drawingContext.Close();
				AddRasterizeOperationToQueue();
			});
		}

		/// <summary>
		/// Draws and fills a triangle on the screen using the selected brush.
		/// </summary>
		/// <param name="x1">
		/// The x co-ordinate of the first point.
		/// </param>
		/// <param name="y1">
		/// The y co-ordinate of the first point.
		/// </param>
		/// <param name="x2">
		/// The x co-ordinate of the second point.
		/// </param>
		/// <param name="y2">
		/// The y co-ordinate of the second point.
		/// </param>
		/// <param name="x3">
		/// The x co-ordinate of the third point.
		/// </param>
		/// <param name="y3">
		/// The y co-ordinate of the third point.
		/// </param>
		public static void FillTriangle(Primitive x1, Primitive y1, Primitive x2, Primitive y2, Primitive x3, Primitive y3)
		{
			VerifyAccess();
			BeginInvoke(delegate
			{
				DrawingContext drawingContext = _mainDrawing.Append();
				PathFigure value = new PathFigure
				{
					StartPoint = new System.Windows.Point(x1, y1),
					IsClosed = true,
					Segments = 
					{
						(PathSegment)new LineSegment(new System.Windows.Point(x2, y2), isStroked: true),
						(PathSegment)new LineSegment(new System.Windows.Point(x3, y3), isStroked: true)
					}
				};
				PathFigureCollection figures = new PathFigureCollection
				{
					value
				};
				PathGeometry pathGeometry = new PathGeometry();
				pathGeometry.Figures = figures;
				pathGeometry.Freeze();
				drawingContext.DrawGeometry(_fillBrush, null, pathGeometry);
				drawingContext.Close();
				AddRasterizeOperationToQueue();
			});
		}

		/// <summary>
		/// Draws a line from one point to another.
		/// </summary>
		/// <param name="x1">
		/// The x co-ordinate of the first point.
		/// </param>
		/// <param name="y1">
		/// The y co-ordinate of the first point.
		/// </param>
		/// <param name="x2">
		/// The x co-ordinate of the second point.
		/// </param>
		/// <param name="y2">
		/// The y co-ordinate of the second point.
		/// </param>
		public static void DrawLine(Primitive x1, Primitive y1, Primitive x2, Primitive y2)
		{
			VerifyAccess();
			BeginInvoke(delegate
			{
				DrawingContext drawingContext = _mainDrawing.Append();
				drawingContext.DrawLine(_pen, new System.Windows.Point((int)x1, (int)y1), new System.Windows.Point((int)x2, (int)y2));
				drawingContext.Close();
				AddRasterizeOperationToQueue();
			});
		}

		/// <summary>
		/// Draws a line of text on the screen at the specified location.
		/// </summary>
		/// <param name="x">
		/// The x co-ordinate of the text start point.
		/// </param>
		/// <param name="y">
		/// The y co-ordinate of the text start point.
		/// </param>
		/// <param name="text">
		/// The text to draw
		/// </param>
		public static void DrawText(Primitive x, Primitive y, Primitive text)
		{
			if (!text.IsEmpty)
			{
				VerifyAccess();
				BeginInvoke(delegate
				{
					DrawingContext drawingContext = _mainDrawing.Append();
					FormattedText formattedText = new FormattedText(text, CultureInfo.CurrentUICulture, FlowDirection.LeftToRight, new Typeface(_fontFamily, _fontStyle, _fontWeight, FontStretches.Normal), _fontSize, _fillBrush);
					drawingContext.DrawText(formattedText, new System.Windows.Point((int)x, (int)y));
					drawingContext.Close();
					AddRasterizeOperationToQueue();
				});
			}
		}

		/// <summary>
		/// Draws a line of text on the screen at the specified location.
		/// </summary>
		/// <param name="x">
		/// The x co-ordinate of the text start point.
		/// </param>
		/// <param name="y">
		/// The y co-ordinate of the text start point.
		/// </param>
		/// <param name="width">
		/// The maximum available width.  This parameter helps define when the text should wrap.
		/// </param>
		/// <param name="text">
		/// The text to draw.
		/// </param>
		public static void DrawBoundText(Primitive x, Primitive y, Primitive width, Primitive text)
		{
			if (!text.IsEmpty)
			{
				VerifyAccess();
				BeginInvoke(delegate
				{
					DrawingContext drawingContext = _mainDrawing.Append();
					drawingContext.DrawText(new FormattedText(text, CultureInfo.CurrentUICulture, FlowDirection.LeftToRight, new Typeface(_fontFamily, _fontStyle, _fontWeight, FontStretches.Normal), _fontSize, _fillBrush)
					{
						MaxTextWidth = width
					}, new System.Windows.Point((int)x, (int)y));
					drawingContext.Close();
					AddRasterizeOperationToQueue();
				});
			}
		}

		/// <summary>
		/// Draws the specified image from memory on to the screen, in the specified size.
		/// </summary>
		/// <param name="imageName">
		/// The name of the image to draw
		/// </param>
		/// <param name="x">
		/// The x co-ordinate of the point to draw the image at.
		/// </param>
		/// <param name="y">
		/// The y co-ordinate of the point to draw the image at.
		/// </param>
		/// <param name="width">
		/// The width to draw the image.
		/// </param>
		/// <param name="height">
		/// The height to draw the image.
		/// </param>
		public static void DrawResizedImage(Primitive imageName, Primitive x, Primitive y, Primitive width, Primitive height)
		{
			if (imageName.IsEmpty)
			{
				return;
			}
			VerifyAccess();
			BitmapSource image = ImageList.GetBitmap(imageName);
			if (image != null)
			{
				BeginInvoke(delegate
				{
					DrawingContext drawingContext = _mainDrawing.Append();
					drawingContext.DrawImage(image, new Rect(x, y, width, height));
					drawingContext.Close();
					AddRasterizeOperationToQueue();
				});
			}
		}

		/// <summary>
		/// Draws the specified image from memory on to the screen.  
		/// </summary>
		/// <param name="imageName">
		/// The name of the image to draw.
		/// </param>
		/// <param name="x">
		/// The x co-ordinate of the point to draw the image at.
		/// </param>
		/// <param name="y">
		/// The y co-ordinate of the point to draw the image at.
		/// </param>
		public static void DrawImage(Primitive imageName, Primitive x, Primitive y)
		{
			if (imageName.IsEmpty)
			{
				return;
			}
			VerifyAccess();
			BitmapSource image = ImageList.GetBitmap(imageName);
			if (image != null)
			{
				BeginInvoke(delegate
				{
					DrawingContext drawingContext = _mainDrawing.Append();
					drawingContext.DrawImage(image, new Rect(x, y, image.PixelWidth, image.PixelHeight));
					drawingContext.Close();
					AddRasterizeOperationToQueue();
				});
			}
		}

		/// <summary>
		/// Draws the pixel specified by the x and y co-ordinates using the specified color.
		/// </summary>
		/// <param name="x">
		/// The x co-ordinate of the pixel.
		/// </param>
		/// <param name="y">
		/// The y co-ordinate of the pixel.
		/// </param>
		/// <param name="color">
		/// The color of the pixel to set.
		/// </param>
		public static void SetPixel(Primitive x, Primitive y, Primitive color)
		{
			VerifyAccess();
			BeginInvoke(delegate
			{
				DrawingContext drawingContext = _mainDrawing.Append();
				drawingContext.DrawRectangle(new SolidColorBrush(GetColorFromString(color)), null, new Rect((int)x, (int)y, 1.0, 1.0));
				drawingContext.Close();
				AddRasterizeOperationToQueue();
			});
		}

		/// <summary>
		/// Gets the color of the pixel at the specified x and y co-ordinates.
		/// </summary>
		/// <param name="x">
		/// The x co-ordinate of the pixel.
		/// </param>
		/// <param name="y">
		/// The y co-ordinate of the pixel.
		/// </param>
		/// <returns>
		/// The color of the pixel.
		/// </returns>
		public static Primitive GetPixel(Primitive x, Primitive y)
		{
			VerifyAccess();
			if ((bool)(x < 0))
			{
				x = 0;
			}
			if ((bool)(y < 0))
			{
				y = 0;
			}
			if ((bool)(x > Width))
			{
				x = Width;
			}
			if ((bool)(y > Height))
			{
				y = Height;
			}
			return (Primitive)InvokeWithReturn(delegate
			{
				Rasterize();
				byte[] array = new byte[4];
				int stride = (int)_renderBitmap.Width * ((_renderBitmap.Format.BitsPerPixel + 7) / 8);
				_renderBitmap.CopyPixels(new Int32Rect(x, y, 1, 1), array, stride, 0);
				return (Primitive)$"#{array[2]:X2}{array[1]:X2}{array[0]:X2}";
			});
		}

		/// <summary>
		/// Gets a valid random color.
		/// </summary>
		/// <returns>
		/// A valid random color.
		/// </returns>
		public static Primitive GetRandomColor()
		{
			if (_random == null)
			{
				_random = new Random((int)DateTime.Now.Ticks);
			}
			return $"#{_random.Next(256):X2}{_random.Next(256):X2}{_random.Next(256):X2}";
		}

		/// <summary>
		/// Constructs a color given the Red, Green and Blue values.
		/// </summary>
		/// <param name="red">
		/// The red component of the Color (0-255).
		/// </param>
		/// <param name="green">
		/// The green component of the color (0-255).
		/// </param>
		/// <param name="blue">
		/// The blue component of the color (0-255).
		/// </param>
		/// <returns>
		/// Returns a color that can be used to set the brush or pen color.
		/// </returns>
		public static Primitive GetColorFromRGB(Primitive red, Primitive green, Primitive blue)
		{
			int num = System.Math.Abs((int)red % 256);
			int num2 = System.Math.Abs((int)green % 256);
			int num3 = System.Math.Abs((int)blue % 256);
			return $"#{num:X2}{num2:X2}{num3:X2}";
		}

		/// <summary>
		/// Clears the window.
		/// </summary>
		public static void Clear()
		{
			VerifyAccess();
			Invoke(delegate
			{
				_mainCanvas.Children.Clear();
				_renderBitmap.Clear();
				_mainDrawing.Children.Clear();
				_objectsMap.Clear();
			});
		}

		/// <summary>
		/// Displays a message box to the user.
		/// </summary>
		/// <param name="text">
		/// The text to be displayed on the message box.
		/// </param>
		/// <param name="title">
		/// The title for the message box.
		/// </param>
		public static void ShowMessage(Primitive text, Primitive title)
		{
			VerifyAccess();
			Invoke(delegate
			{
				MessageBox.Show(_window, text, title);
			});
		}

		private static void CreateWindow()
		{
			lock (_syncLock)
			{
				Invoke(delegate
				{
					_window = new Window();
					_windowCreated = true;
					_window.Title = "Small Basic Graphics Window";
					if (_backgroundBrush == null)
					{
						_backgroundBrush = System.Windows.Media.Brushes.White;
					}
					_window.Background = _backgroundBrush;
					if (_fillBrush == null)
					{
						_fillBrush = System.Windows.Media.Brushes.CornflowerBlue;
					}
					if (_pen == null)
					{
						_pen = new System.Windows.Media.Pen(System.Windows.Media.Brushes.Black, 2.0);
					}
					if (_fontFamily == null)
					{
						_fontFamily = new System.Windows.Media.FontFamily("Tahoma");
					}
					_window.Height = 480.0;
					_window.Width = 640.0;
					if (!_isHidden)
					{
						_window.Show();
					}
					_window.SourceInitialized += delegate
					{
						IntPtr handle = ((HwndSource)PresentationSource.FromVisual(_window)).Handle;
						uint windowLong = NativeHelper.GetWindowLong(handle, -16);
						windowLong = (windowLong & 0xFFFEFFFFu) | 0x20000u;
						NativeHelper.SetWindowLong(handle, -16, windowLong);
						RECT rECT = default(RECT);
						rECT.Left = 0;
						rECT.Top = 0;
						rECT.Right = (int)_window.Width;
						rECT.Bottom = (int)_window.Height;
						RECT lpRect = rECT;
						NativeHelper.AdjustWindowRect(ref lpRect, windowLong, bMenu: false);
						_window.Width = lpRect.Right - lpRect.Left;
						_window.Height = lpRect.Bottom - lpRect.Top;
						NativeHelper.SetForegroundWindow(handle);
					};
					SetWindowContent();
					SignupForWindowEvents();
					_window.UpdateLayout();
				});
			}
		}

		private static void SignupForWindowEvents()
		{
			_window.SizeChanged += WindowSizeChanged;
			_window.Closing += WindowClosing;
			_window.MouseDown += delegate
			{
				GraphicsWindow._mouseDown?.Invoke();
			};
			_window.MouseUp += delegate
			{
				GraphicsWindow._mouseUp?.Invoke();
			};
			_window.MouseMove += delegate(object sender, MouseEventArgs e)
			{
				SmallBasicCallback mouseMove = GraphicsWindow._mouseMove;
				System.Windows.Point position = e.GetPosition(_mainCanvas);
				_mouseX = position.X;
				_mouseY = position.Y;
				mouseMove?.Invoke();
			};
			_window.KeyDown += delegate(object sender, KeyEventArgs e)
			{
				SmallBasicCallback temp3 = GraphicsWindow._keyDown;
				_lastKey = e.Key;
				if (temp3 != null)
				{
					BeginInvoke(delegate
					{
						temp3();
					});
				}
			};
			_window.TextInput += delegate(object sender, TextCompositionEventArgs e)
			{
				SmallBasicCallback temp2 = GraphicsWindow._textInput;
				_lastText = e.Text;
				if (temp2 != null)
				{
					BeginInvoke(delegate
					{
						temp2();
					});
				}
			};
			_window.KeyUp += delegate(object sender, KeyEventArgs e)
			{
				SmallBasicCallback temp = GraphicsWindow._keyUp;
				_lastKey = e.Key;
				if (temp != null)
				{
					BeginInvoke(delegate
					{
						temp();
					});
				}
			};
		}

		private static void SetWindowContent()
		{
			_mainCanvas = new Canvas();
			_mainCanvas.HorizontalAlignment = HorizontalAlignment.Stretch;
			_mainCanvas.VerticalAlignment = VerticalAlignment.Stretch;
			_renderBitmap = new RenderTargetBitmap((int)_window.Width + 120, (int)_window.Height + 120, 96.0, 96.0, PixelFormats.Default);
			System.Windows.Controls.Image image = new System.Windows.Controls.Image();
			image.Source = _renderBitmap;
			image.Stretch = Stretch.None;
			_bitmapContainer = image;
			_mainDrawing = new DrawingGroup();
			_visualContainer = new VisualContainer(_mainDrawing);
			Grid grid = new Grid();
			grid.Children.Add(_bitmapContainer);
			grid.Children.Add(_visualContainer);
			grid.Children.Add(_mainCanvas);
			_window.Content = grid;
		}

		private static void WindowClosing(object sender, CancelEventArgs e)
		{
			SmallBasicApplication.End();
		}

		private static void WindowSizeChanged(object sender, SizeChangedEventArgs e)
		{
			if (e.NewSize.Width > _renderBitmap.Width || e.NewSize.Height > _renderBitmap.Height)
			{
				RenderTargetBitmap renderBitmap = _renderBitmap;
				_renderBitmap = new RenderTargetBitmap((int)e.NewSize.Width + 120, (int)e.NewSize.Height + 120, 96.0, 96.0, PixelFormats.Default);
				DrawingVisual drawingVisual = new DrawingVisual();
				DrawingContext drawingContext = drawingVisual.RenderOpen();
				drawingContext.DrawImage(renderBitmap, new Rect(0.0, 0.0, renderBitmap.Width, renderBitmap.Height));
				drawingContext.Close();
				_renderBitmap.Render(drawingVisual);
				_bitmapContainer.Width = _renderBitmap.Width;
				_bitmapContainer.Height = _renderBitmap.Height;
				_bitmapContainer.Source = _renderBitmap;
				_bitmapContainer.Stretch = Stretch.None;
			}
		}

		internal static void VerifyAccess()
		{
			if (!_windowCreated)
			{
				CreateWindow();
				if (!_isHidden)
				{
					Show();
				}
			}
		}

		internal static void BeginInvoke(InvokeHelper invokeDelegate)
		{
			SmallBasicApplication.BeginInvoke(invokeDelegate);
		}

		internal static void Invoke(InvokeHelper invokeDelegate)
		{
			SmallBasicApplication.Invoke(invokeDelegate);
		}

		internal static object InvokeWithReturn(InvokeHelperWithReturn invokeDelegate)
		{
			return SmallBasicApplication.InvokeWithReturn(invokeDelegate);
		}

		internal static void AddShape(string name, FrameworkElement shape)
		{
			VerifyAccess();
			shape.Name = name;
			_objectsMap[name] = shape;
			_mainCanvas.Children.Add(shape);
		}

		internal static void AddControl(string name, Control control)
		{
			VerifyAccess();
			control.Foreground = _fillBrush;
			control.FontFamily = _fontFamily;
			control.FontStyle = _fontStyle;
			control.FontSize = _fontSize;
			control.FontWeight = _fontWeight;
			AddShape(name, control);
		}

		internal static void RemoveShape(Primitive name)
		{
			UIElement shape = null;
			if (_objectsMap.TryGetValue(name, out shape))
			{
				_objectsMap.Remove(name);
				Invoke(delegate
				{
					_mainCanvas.Children.Remove(shape);
				});
			}
		}

		internal static bool ContainsShape(string name)
		{
			return _objectsMap.ContainsKey(name);
		}

		internal static void AddRasterizeOperationToQueue()
		{
			if (_mainDrawing.Children.Count > 100)
			{
				Rasterize();
			}
		}

		private static void Rasterize()
		{
			if (_mainDrawing.Children.Count != 0)
			{
				_renderBitmap.Render(_visualContainer);
				_mainDrawing.Children.Clear();
			}
		}

		internal static void DoubleAnimateProperty(IAnimatable animatable, DependencyProperty property, double end, double timespan)
		{
			double num = (double)((DependencyObject)animatable).GetValue(property);
			if (double.IsNaN(num))
			{
				num = 0.0;
			}
			DoubleAnimation doubleAnimation = new DoubleAnimation(num, end, new Duration(TimeSpan.FromMilliseconds(timespan)));
			doubleAnimation.FillBehavior = FillBehavior.HoldEnd;
			doubleAnimation.DecelerationRatio = 0.2;
			DoubleAnimation animation = doubleAnimation;
			animatable.BeginAnimation(property, animation);
		}

		internal static System.Windows.Media.Color GetColorFromString(string color)
		{
			try
			{
				return (System.Windows.Media.Color)System.Windows.Media.ColorConverter.ConvertFromString(color);
			}
			catch (Exception)
			{
			}
			return Colors.Black;
		}

		internal static string GetStringFromColor(System.Windows.Media.Color color)
		{
			System.Drawing.Color c = System.Drawing.Color.FromArgb(color.A, color.R, color.G, color.B);
			return ColorTranslator.ToHtml(c);
		}

		internal static void SetCursor(Cursor cursor)
		{
			VerifyAccess();
			Invoke(delegate
			{
				_window.Cursor = cursor;
			});
		}
	}
}
