using System;
using System.Collections.Generic;
using System.Windows.Media.Imaging;
using Microsoft.SmallBasic.Library.Internal;

namespace Microsoft.SmallBasic.Library
{
	/// <summary>
	/// This class helps to load and store images in memory.
	/// </summary>
	[SmallBasicType]
	public sealed class ImageList
	{
		private static Dictionary<string, BitmapSource> _savedImages = new Dictionary<string, BitmapSource>();

		/// <summary>
		/// Loads an image from a file or the Internet into memory.
		/// </summary>
		/// <param name="fileNameOrUrl">
		/// The file name to load the image from.  This could be a local file or a URL to the Internet location.
		/// </param>
		/// <returns>
		/// Returns the name of the image that was loaded.
		/// </returns>
		public static Primitive LoadImage(Primitive fileNameOrUrl)
		{
			if (fileNameOrUrl.IsEmpty)
			{
				return "";
			}
			string text = Shapes.GenerateNewName("ImageList");
			_savedImages[text] = LoadImageFromFile(fileNameOrUrl);
			return text;
		}

		/// <summary>
		/// Gets the width of the stored image.
		/// </summary>
		/// <param name="imageName">
		/// The name of the image in memory.
		/// </param>
		/// <returns>
		/// The width of the specified image.
		/// </returns>
		public static Primitive GetWidthOfImage(Primitive imageName)
		{
			BitmapSource image = GetBitmap(imageName);
			if (image == null)
			{
				return 0;
			}
			return (Primitive)GraphicsWindow.InvokeWithReturn(() => (Primitive)image.PixelWidth);
		}

		/// <summary>
		/// Gets the height of the stored image.
		/// </summary>
		/// <param name="imageName">
		/// The name of the image in memory.
		/// </param>
		/// <returns>
		/// The height of the specified image.
		/// </returns>
		public static Primitive GetHeightOfImage(Primitive imageName)
		{
			BitmapSource image = GetBitmap(imageName);
			if (image == null)
			{
				return 0;
			}
			return (Primitive)GraphicsWindow.InvokeWithReturn(() => (Primitive)image.PixelHeight);
		}

		internal static BitmapSource GetBitmap(Primitive imageName)
		{
			BitmapSource value = null;
			if (!imageName.IsEmpty)
			{
				_savedImages.TryGetValue(imageName, out value);
			}
			if (value == null)
			{
				value = LoadImageFromFile(imageName);
			}
			return value;
		}

		internal static BitmapImage LoadImageFromFile(Primitive fileNameOrUrl)
		{
			Primitive localFileName = Network.GetLocalFile(fileNameOrUrl);
			return (BitmapImage)SmallBasicApplication.InvokeWithReturn(delegate
			{
				try
				{
					return new BitmapImage(new Uri(localFileName));
				}
				catch (Exception)
				{
					return null;
				}
			});
		}
	}
}
