using System;

namespace Microsoft.SmallBasic.Library
{
	/// <summary>
	/// The Math class provides lots of useful mathematics related methods
	/// </summary>
	[SmallBasicType]
	public sealed class Math
	{
		private static Random _random;

		/// <summary>
		/// Gets the value of Pi
		/// </summary>
		public static Primitive Pi => System.Math.PI;

		/// <summary>
		/// Gets the absolute value of the given number.  For example, -32.233 will return 32.233.
		/// </summary>
		/// <param name="number">
		/// The number to get the absolute value for.
		/// </param>
		/// <returns>
		/// The absolute value of the given number.
		/// </returns>
		public static Primitive Abs(Primitive number)
		{
			return DoubleToDecimal(System.Math.Abs((double)number));
		}

		/// <summary>
		/// Returns the smallest integer that is greater than or equal to the argument.  It rounds up the integer value.
		/// For example, 32.233 will return 33. Also, 44 will return 44.
		/// </summary>
		/// <param name="number">
		/// The number whose ceiling is required.
		/// </param>
		/// <returns>
		/// The ceiling value of the given number.
		/// </returns>
		public static Primitive Ceiling(Primitive number)
		{
			return DoubleToDecimal(System.Math.Ceiling((double)number));
		}

		/// <summary>
		/// Returns the largest integer that is less than or equal to the argument.  It rounds down the integer value.
		/// For example, 32.233 will return 32. Also, 44 will return 44.
		/// </summary>
		/// <param name="number">
		/// The number whose floor value is required.
		/// </param>
		/// <returns>
		/// The floor value of the given number.
		/// </returns>
		public static Primitive Floor(Primitive number)
		{
			return DoubleToDecimal(System.Math.Floor((double)number));
		}

		/// <summary>
		/// Gets the natural logarithm value of the given number.
		/// </summary>
		/// <param name="number">
		/// The number whose natural logarithm value is required.
		/// </param>
		/// <returns>
		/// The natural log value of the given number.
		/// </returns>
		public static Primitive NaturalLog(Primitive number)
		{
			return DoubleToDecimal(System.Math.Log(number));
		}

		/// <summary>
		/// Gets the logarithm (base 10) value of the given number.
		/// </summary>
		/// <param name="number">
		/// The number whose logarithm value is required
		/// </param>
		/// <returns>
		/// The log value of the given number
		/// </returns>
		public static Primitive Log(Primitive number)
		{
			return DoubleToDecimal(System.Math.Log10(number));
		}

		/// <summary>
		/// Gets the cosine of the given angle in radians.
		/// </summary>
		/// <param name="angle">
		/// The angle whose cosine is needed (in radians).
		/// </param>
		/// <returns>
		/// The cosine of the given angle.
		/// </returns>
		public static Primitive Cos(Primitive angle)
		{
			return DoubleToDecimal(System.Math.Cos(angle));
		}

		/// <summary>
		/// Gets the sine of the given angle in radians.
		/// </summary>
		/// <param name="angle">
		/// The angle whose sine is needed (in radians)
		/// </param>
		/// <returns>
		/// The sine of the given angle
		/// </returns>
		public static Primitive Sin(Primitive angle)
		{
			return DoubleToDecimal(System.Math.Sin(angle));
		}

		/// <summary>
		/// Gets the tangent of the given angle in radians.
		/// </summary>
		/// <param name="angle">
		/// The angle whose tangent is needed (in radians).
		/// </param>
		/// <returns>
		/// The tangent of the given angle.
		/// </returns>
		public static Primitive Tan(Primitive angle)
		{
			return DoubleToDecimal(System.Math.Tan(angle));
		}

		/// <summary>
		/// Gets the angle in radians, given the sin value.
		/// </summary>
		/// <param name="sinValue">
		/// The sine value whose angle is needed.
		/// </param>
		/// <returns>
		/// The angle (in radians) for the given sine Value.
		/// </returns>
		public static Primitive ArcSin(Primitive sinValue)
		{
			return DoubleToDecimal(System.Math.Asin(sinValue));
		}

		/// <summary>
		/// Gets the angle in radians, given the cosine value.
		/// </summary>
		/// <param name="cosValue">
		/// The cosine value whose angle is needed.
		/// </param>
		/// <returns>
		/// The angle (in radians) for the given cosine Value.
		/// </returns>
		public static Primitive ArcCos(Primitive cosValue)
		{
			return DoubleToDecimal(System.Math.Acos(cosValue));
		}

		/// <summary>
		/// Gets the angle in radians, given the tangent value.
		/// </summary>
		/// <param name="tanValue">
		/// The tangent value whose angle is needed.
		/// </param>
		/// <returns>
		/// The angle (in radians) for the given tangent Value.
		/// </returns>
		public static Primitive ArcTan(Primitive tanValue)
		{
			return DoubleToDecimal(System.Math.Atan(tanValue));
		}

		/// <summary>
		/// Converts a given angle in radians to degrees.
		/// </summary>
		/// <param name="angle">
		/// The angle in radians.
		/// </param>
		/// <returns>
		/// The converted angle in degrees.
		/// </returns>
		public static Primitive GetDegrees(Primitive angle)
		{
			return DoubleToDecimal(180.0 * (double)angle / System.Math.PI % 360.0);
		}

		/// <summary>
		/// Converts a given angle in degrees to radians.
		/// </summary>
		/// <param name="angle">
		/// The angle in degrees.
		/// </param>
		/// <returns>
		/// The converted angle in radians.
		/// </returns>
		public static Primitive GetRadians(Primitive angle)
		{
			return DoubleToDecimal((double)angle % 360.0 * System.Math.PI / 180.0);
		}

		/// <summary>
		/// Gets the square root of a given number.
		/// </summary>
		/// <param name="number">
		/// The number whose square root value is needed.
		/// </param>
		/// <returns>
		/// The square root value of the given number.
		/// </returns>
		public static Primitive SquareRoot(Primitive number)
		{
			return DoubleToDecimal(System.Math.Sqrt(number));
		}

		/// <summary>
		/// Raises the base number to the specified power.
		/// </summary>
		/// <param name="baseNumber">
		/// The number to be raised to the exponent power.
		/// </param>
		/// <param name="exponent">
		/// The power to raise the base number.
		/// </param>
		/// <returns>
		/// The base number raised to the specified exponent.
		/// </returns>
		public static Primitive Power(Primitive baseNumber, Primitive exponent)
		{
			return DoubleToDecimal(System.Math.Pow(baseNumber, exponent));
		}

		/// <summary>
		/// Rounds a given number to the nearest integer.  For example 32.233 will be rounded to 32.0 while 
		/// 32.566 will be rounded to 33.
		/// </summary>
		/// <param name="number">
		/// The number whose approximation is required.
		/// </param>
		/// <returns>
		/// The rounded value of the given number.
		/// </returns>
		public static Primitive Round(Primitive number)
		{
			return DoubleToDecimal(System.Math.Round((double)number));
		}

		/// <summary>
		/// Compares two numbers and returns the greater of the two.
		/// </summary>
		/// <param name="number1">
		/// The first of the two numbers to compare.
		/// </param>
		/// <param name="number2">
		/// The second of the two numbers to compare.
		/// </param>
		/// <returns>
		/// The greater value of the two numbers.
		/// </returns>
		public static Primitive Max(Primitive number1, Primitive number2)
		{
			return DoubleToDecimal(System.Math.Max((double)number1, (double)number2));
		}

		/// <summary>
		/// Compares two numbers and returns the smaller of the two.
		/// </summary>
		/// <param name="number1">
		/// The first of the two numbers to compare.
		/// </param>
		/// <param name="number2">
		/// The second of the two numbers to compare.
		/// </param>
		/// <returns>
		/// The smaller value of the two numbers.
		/// </returns>
		public static Primitive Min(Primitive number1, Primitive number2)
		{
			return DoubleToDecimal(System.Math.Min((double)number1, (double)number2));
		}

		/// <summary>
		/// Divides the first number by the second and returns the remainder.
		/// </summary>
		/// <param name="dividend">
		/// The number to divide.
		/// </param>
		/// <param name="divisor">
		/// The number that divides.
		/// </param>
		/// <returns>
		/// The remainder after the division.
		/// </returns>
		public static Primitive Remainder(Primitive dividend, Primitive divisor)
		{
			return (double)DoubleToDecimal(dividend) % (double)divisor;
		}

		/// <summary>
		/// Gets a random number between 1 and the specified maxNumber (inclusive).
		/// </summary>
		/// <param name="maxNumber">
		/// The maximum number for the requested random value.
		/// </param>
		/// <returns>
		/// A Random number that is less than or equal to the specified max.
		/// </returns>
		public static Primitive GetRandomNumber(Primitive maxNumber)
		{
			if ((bool)(maxNumber < 1))
			{
				maxNumber = 1;
			}
			if (_random == null)
			{
				_random = new Random((int)DateTime.Now.Ticks);
			}
			return _random.Next(maxNumber) + 1;
		}

		/// <summary>
		/// Handles double to decimal conversion
		/// </summary>
		/// <param name="number">
		/// The input number
		/// </param>
		/// <returns>
		/// The output number
		/// </returns>
		private static Primitive DoubleToDecimal(double number)
		{
			if (double.IsNaN(number))
			{
				return new Primitive(0);
			}
			if (double.IsPositiveInfinity(number))
			{
				return new Primitive(decimal.MaxValue);
			}
			if (double.IsNegativeInfinity(number))
			{
				return new Primitive(decimal.MinValue);
			}
			return new Primitive(number);
		}
	}
}
