using System;
using System.Globalization;
using System.IO;
using System.Net;

namespace Microsoft.SmallBasic.Library
{
	/// <summary>
	/// This helper class provides network access methods
	/// </summary>
	[SmallBasicType]
	public sealed class Network
	{
		/// <summary>
		/// Downloads a file from the network to a local temporary file.
		/// </summary>
		/// <param name="url">
		/// The URL of the file on the network.
		/// </param>
		/// <returns>
		/// A local file name that the remote file was downloaded as.
		/// </returns>
		public static Primitive DownloadFile(Primitive url)
		{
			if (url.IsEmpty)
			{
				return url;
			}
			string tempFileName = Path.GetTempFileName();
			Stream stream = null;
			Stream stream2 = null;
			WebResponse webResponse = null;
			try
			{
				WebRequest webRequest = WebRequest.Create(url);
				webResponse = webRequest.GetResponse();
				stream = System.IO.File.Open(tempFileName, FileMode.OpenOrCreate, FileAccess.Write, FileShare.Read);
				byte[] buffer = new byte[16384];
				long num = webResponse.ContentLength;
				stream2 = webResponse.GetResponseStream();
				while (num > 0)
				{
					int num2 = stream2.Read(buffer, 0, 16384);
					stream.Write(buffer, 0, num2);
					num -= num2;
				}
			}
			catch (Exception)
			{
				return "";
			}
			finally
			{
				stream?.Close();
				stream2?.Close();
				webResponse?.Close();
			}
			return tempFileName;
		}

		/// <summary>
		/// Gets the contents of a specified web page.
		/// </summary>
		/// <param name="url">
		/// The URL of the web page
		/// </param>
		/// <returns>
		/// The contents of the specified web page.
		/// </returns>
		public static Primitive GetWebPageContents(Primitive url)
		{
			if (url.IsEmpty)
			{
				return url;
			}
			return GetWebPageContents((string)url);
		}

		internal static Primitive GetLocalFile(Primitive fileNameOrUrl)
		{
			if (fileNameOrUrl.IsEmpty)
			{
				return fileNameOrUrl;
			}
			if (Uri.TryCreate(fileNameOrUrl, UriKind.RelativeOrAbsolute, out var result) && result.IsAbsoluteUri)
			{
				if (result.Scheme.ToLower(CultureInfo.InvariantCulture) == "file")
				{
					return fileNameOrUrl;
				}
				return DownloadFile(fileNameOrUrl);
			}
			return fileNameOrUrl;
		}

		internal static string GetWebPageContents(string url)
		{
			StreamReader streamReader = null;
			WebResponse webResponse = null;
			string result = "";
			try
			{
				WebRequest webRequest = WebRequest.Create(url);
				webResponse = webRequest.GetResponse();
				streamReader = new StreamReader(webResponse.GetResponseStream());
				result = streamReader.ReadToEnd();
				return result;
			}
			catch (Exception)
			{
				return result;
			}
			finally
			{
				streamReader?.Close();
				webResponse?.Close();
			}
		}
	}
}
