using System;
using System.Collections.Generic;
using System.Globalization;
using System.Text;

namespace Microsoft.SmallBasic.Library
{
	/// <summary>
	/// The primitive type representing either text or number.
	/// </summary>
	public struct Primitive
	{
		private class PrimitiveComparer : IEqualityComparer<Primitive>
		{
			private static PrimitiveComparer _instance = new PrimitiveComparer();

			public static PrimitiveComparer Instance => _instance;

			private PrimitiveComparer()
			{
			}

			public bool Equals(Primitive x, Primitive y)
			{
				return string.Equals(x.AsString, y.AsString, StringComparison.InvariantCultureIgnoreCase);
			}

			public int GetHashCode(Primitive obj)
			{
				return obj.GetHashCode();
			}
		}

		private string _primitive;

		private decimal? _primitiveAsDecimal;

		private Dictionary<Primitive, Primitive> _arrayMap;

		public Primitive this[Primitive index]
		{
			get
			{
				if ((bool)ContainsKey(index))
				{
					return _arrayMap[index];
				}
				return "";
			}
			set
			{
				Primitive primitive = SetArrayValue(value, this, index);
				_primitive = primitive._primitive;
				_arrayMap = primitive._arrayMap;
				_primitiveAsDecimal = primitive._primitiveAsDecimal;
			}
		}

		internal string AsString
		{
			get
			{
				if (_primitive != null)
				{
					return _primitive;
				}
				if (_primitiveAsDecimal.HasValue)
				{
					_primitive = _primitiveAsDecimal.Value.ToString();
				}
				if (_arrayMap != null)
				{
					StringBuilder stringBuilder = new StringBuilder();
					foreach (KeyValuePair<Primitive, Primitive> item in _arrayMap)
					{
						stringBuilder.AppendFormat("{0}={1};", Escape(item.Key), Escape(item.Value));
					}
					_primitive = stringBuilder.ToString();
				}
				return _primitive;
			}
		}

		internal bool IsArray
		{
			get
			{
				ConstructArrayMap();
				return _arrayMap.Count > 0;
			}
		}

		internal bool IsEmpty
		{
			get
			{
				if (string.IsNullOrEmpty(_primitive) && !_primitiveAsDecimal.HasValue)
				{
					if (_arrayMap != null)
					{
						return _arrayMap.Count == 0;
					}
					return true;
				}
				return false;
			}
		}

		internal bool IsNumber
		{
			get
			{
				decimal result = 0m;
				if (!_primitiveAsDecimal.HasValue)
				{
					return decimal.TryParse(AsString, NumberStyles.Float, CultureInfo.InvariantCulture, out result);
				}
				return true;
			}
		}

		public Primitive(string primitiveText)
		{
			if (primitiveText == null)
			{
				throw new ArgumentNullException("primitiveText");
			}
			_primitive = primitiveText;
			_primitiveAsDecimal = null;
			_arrayMap = null;
		}

		public Primitive(int primitiveInteger)
		{
			_primitiveAsDecimal = primitiveInteger;
			_primitive = null;
			_arrayMap = null;
		}

		public Primitive(decimal primitiveDecimal)
		{
			_primitiveAsDecimal = primitiveDecimal;
			_primitive = null;
			_arrayMap = null;
		}

		public Primitive(float primitiveFloat)
			: this((decimal)primitiveFloat)
		{
		}

		public Primitive(double primitiveDouble)
			: this((decimal)primitiveDouble)
		{
		}

		public Primitive(short primitiveShort)
			: this((int)primitiveShort)
		{
		}

		public Primitive(long primitiveLong)
		{
			_primitiveAsDecimal = primitiveLong;
			_primitive = null;
			_arrayMap = null;
		}

		public Primitive(object primitiveObject)
		{
			_primitive = primitiveObject.ToString();
			_primitiveAsDecimal = null;
			_arrayMap = null;
		}

		public Primitive(bool primitiveBool)
		{
			_primitiveAsDecimal = null;
			if (primitiveBool)
			{
				_primitive = "True";
			}
			else
			{
				_primitive = "False";
			}
			_arrayMap = null;
		}

		public Primitive Append(Primitive primitive)
		{
			return new Primitive(AsString + primitive.AsString);
		}

		public Primitive Add(Primitive addend)
		{
			decimal? num = TryGetAsDecimal();
			decimal? num2 = addend.TryGetAsDecimal();
			if (num.HasValue && num2.HasValue)
			{
				return num.Value + num2.Value;
			}
			return AsString + addend.AsString;
		}

		public Primitive ContainsKey(Primitive key)
		{
			ConstructArrayMap();
			return _arrayMap.ContainsKey(key);
		}

		public Primitive ContainsValue(Primitive value)
		{
			ConstructArrayMap();
			return _arrayMap.ContainsValue(value);
		}

		public Primitive GetAllIndices()
		{
			ConstructArrayMap();
			Dictionary<Primitive, Primitive> dictionary = new Dictionary<Primitive, Primitive>(_arrayMap.Count, PrimitiveComparer.Instance);
			int num = 1;
			foreach (Primitive key in _arrayMap.Keys)
			{
				dictionary[num] = key;
				num++;
			}
			return ConvertFromMap(dictionary);
		}

		public Primitive GetItemCount()
		{
			ConstructArrayMap();
			return _arrayMap.Count;
		}

		public Primitive Subtract(Primitive addend)
		{
			return new Primitive(GetAsDecimal() - addend.GetAsDecimal());
		}

		public Primitive Multiply(Primitive multiplicand)
		{
			return new Primitive(GetAsDecimal() * multiplicand.GetAsDecimal());
		}

		public Primitive Divide(Primitive divisor)
		{
			divisor.GetAsDecimal();
			return new Primitive(GetAsDecimal() / divisor.GetAsDecimal());
		}

		public bool LessThan(Primitive comparer)
		{
			return GetAsDecimal() < comparer.GetAsDecimal();
		}

		public bool GreaterThan(Primitive comparer)
		{
			return GetAsDecimal() > comparer.GetAsDecimal();
		}

		public bool LessThanOrEqualTo(Primitive comparer)
		{
			return GetAsDecimal() <= comparer.GetAsDecimal();
		}

		public bool GreaterThanOrEqualTo(Primitive comparer)
		{
			return GetAsDecimal() >= comparer.GetAsDecimal();
		}

		public bool EqualTo(Primitive comparer)
		{
			return Equals(comparer);
		}

		public bool NotEqualTo(Primitive comparer)
		{
			return !Equals(comparer);
		}

		public override bool Equals(object obj)
		{
			if (AsString == null)
			{
				_primitive = "";
			}
			if (obj is Primitive)
			{
				Primitive primitive = (Primitive)obj;
				if (primitive.AsString != null && primitive.AsString.ToLower(CultureInfo.InvariantCulture) == "true" && AsString != null && AsString.ToLower(CultureInfo.InvariantCulture) == "true")
				{
					return true;
				}
				if (IsNumber && primitive.IsNumber)
				{
					return GetAsDecimal() == primitive.GetAsDecimal();
				}
				return AsString == primitive.AsString;
			}
			return false;
		}

		public override int GetHashCode()
		{
			if (AsString == null)
			{
				_primitive = "";
			}
			return AsString.ToUpper(CultureInfo.InvariantCulture).GetHashCode();
		}

		public override string ToString()
		{
			return AsString;
		}

		private void ConstructArrayMap()
		{
			if (_arrayMap != null)
			{
				return;
			}
			_arrayMap = new Dictionary<Primitive, Primitive>(PrimitiveComparer.Instance);
			if (IsEmpty)
			{
				return;
			}
			char[] source = AsString.ToCharArray();
			int index = 0;
			while (true)
			{
				string value = Unescape(source, ref index);
				if (string.IsNullOrEmpty(value))
				{
					break;
				}
				string text = Unescape(source, ref index);
				if (text == null)
				{
					break;
				}
				_arrayMap[value] = text;
			}
		}

		internal decimal? TryGetAsDecimal()
		{
			if (IsEmpty)
			{
				return null;
			}
			if (_primitiveAsDecimal.HasValue)
			{
				return _primitiveAsDecimal;
			}
			decimal result = 0m;
			if (decimal.TryParse(AsString, NumberStyles.Float, CultureInfo.InvariantCulture, out result))
			{
				_primitiveAsDecimal = result;
			}
			return _primitiveAsDecimal;
		}

		internal decimal GetAsDecimal()
		{
			if (IsEmpty)
			{
				return 0m;
			}
			if (_primitiveAsDecimal.HasValue)
			{
				return _primitiveAsDecimal.Value;
			}
			decimal result = 0m;
			if (decimal.TryParse(AsString, NumberStyles.Float, CultureInfo.InvariantCulture, out result))
			{
				_primitiveAsDecimal = result;
			}
			return result;
		}

		public static bool ConvertToBoolean(Primitive primitive)
		{
			return primitive;
		}

		public static implicit operator string(Primitive primitive)
		{
			if (primitive.AsString == null)
			{
				return "";
			}
			return primitive.AsString;
		}

		public static implicit operator int(Primitive primitive)
		{
			return (int)primitive.GetAsDecimal();
		}

		public static implicit operator float(Primitive primitive)
		{
			return (float)primitive.GetAsDecimal();
		}

		public static implicit operator double(Primitive primitive)
		{
			return (double)primitive.GetAsDecimal();
		}

		public static implicit operator bool(Primitive primitive)
		{
			if (primitive.AsString != null && primitive.AsString.Equals("true", StringComparison.InvariantCultureIgnoreCase))
			{
				return true;
			}
			return false;
		}

		public static Primitive operator ==(Primitive primitive1, Primitive primitive2)
		{
			return primitive1.Equals(primitive2);
		}

		public static Primitive operator !=(Primitive primitive1, Primitive primitive2)
		{
			return !primitive1.Equals(primitive2);
		}

		public static Primitive operator >(Primitive primitive1, Primitive primitive2)
		{
			return primitive1.GreaterThan(primitive2);
		}

		public static Primitive operator >=(Primitive primitive1, Primitive primitive2)
		{
			return primitive1.GreaterThanOrEqualTo(primitive2);
		}

		public static Primitive operator <(Primitive primitive1, Primitive primitive2)
		{
			return primitive1.LessThan(primitive2);
		}

		public static Primitive operator <=(Primitive primitive1, Primitive primitive2)
		{
			return primitive1.LessThanOrEqualTo(primitive2);
		}

		public static Primitive operator +(Primitive primitive1, Primitive primitive2)
		{
			return primitive1.Add(primitive2);
		}

		public static Primitive operator -(Primitive primitive1, Primitive primitive2)
		{
			return primitive1.Subtract(primitive2);
		}

		public static Primitive operator *(Primitive primitive1, Primitive primitive2)
		{
			return primitive1.Multiply(primitive2);
		}

		public static Primitive operator /(Primitive primitive1, Primitive primitive2)
		{
			return primitive1.Divide(primitive2);
		}

		public static Primitive operator -(Primitive primitive)
		{
			return -primitive.GetAsDecimal();
		}

		public static Primitive operator |(Primitive primitive1, Primitive primitive2)
		{
			return op_Or(primitive1, primitive2);
		}

		public static Primitive operator &(Primitive primitive1, Primitive primitive2)
		{
			return op_And(primitive1, primitive2);
		}

		public static bool operator true(Primitive primitive)
		{
			return primitive;
		}

		public static bool operator false(Primitive primitive)
		{
			return !primitive;
		}

		public static Primitive op_And(Primitive primitive1, Primitive primitive2)
		{
			return (bool)primitive1 && (bool)primitive2;
		}

		public static Primitive op_Or(Primitive primitive1, Primitive primitive2)
		{
			return (bool)primitive1 || (bool)primitive2;
		}

		public static implicit operator Primitive(int value)
		{
			return new Primitive(value);
		}

		public static implicit operator Primitive(bool value)
		{
			return new Primitive(value);
		}

		public static implicit operator Primitive(string value)
		{
			if (value == null)
			{
				return new Primitive("");
			}
			return new Primitive(value);
		}

		public static implicit operator Primitive(double value)
		{
			return new Primitive((decimal)value);
		}

		public static implicit operator Primitive(decimal value)
		{
			return new Primitive(value);
		}

		public static implicit operator Primitive(DateTime value)
		{
			return new Primitive(value);
		}

		public static Primitive GetArrayValue(Primitive array, Primitive indexer)
		{
			array.ConstructArrayMap();
			if (!array._arrayMap.TryGetValue(indexer, out var value))
			{
				value = default(Primitive);
			}
			return value;
		}

		public static Primitive SetArrayValue(Primitive value, Primitive array, Primitive indexer)
		{
			array.ConstructArrayMap();
			Dictionary<Primitive, Primitive> dictionary = new Dictionary<Primitive, Primitive>(array._arrayMap, PrimitiveComparer.Instance);
			if (value.IsEmpty)
			{
				dictionary.Remove(indexer);
			}
			else
			{
				dictionary[indexer] = value;
			}
			return ConvertFromMap(dictionary);
		}

		public static Primitive ConvertFromMap(Dictionary<Primitive, Primitive> map)
		{
			Primitive result = default(Primitive);
			result._primitive = null;
			result._primitiveAsDecimal = null;
			result._arrayMap = map;
			return result;
		}

		private static string Escape(string value)
		{
			StringBuilder stringBuilder = new StringBuilder();
			foreach (char c in value)
			{
				switch (c)
				{
				case '=':
					stringBuilder.Append("\\=");
					break;
				case ';':
					stringBuilder.Append("\\;");
					break;
				case '\\':
					stringBuilder.Append("\\\\");
					break;
				default:
					stringBuilder.Append(c);
					break;
				}
			}
			return stringBuilder.ToString();
		}

		private static string Unescape(char[] source, ref int index)
		{
			bool flag = false;
			bool flag2 = true;
			int num = source.Length;
			StringBuilder stringBuilder = new StringBuilder();
			while (index < num)
			{
				char c = source[index];
				index++;
				if (!flag)
				{
					if (c == '\\')
					{
						flag = true;
						continue;
					}
					if (c == '=' || c == ';')
					{
						break;
					}
				}
				else
				{
					flag = false;
				}
				flag2 = false;
				stringBuilder.Append(c);
			}
			if (flag2)
			{
				return null;
			}
			return stringBuilder.ToString();
		}
	}
}
