using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace Microsoft.SmallBasic.Library
{
	/// <summary>
	/// The Shape object allows you to add, move and rotate shapes to the Graphics window.
	/// </summary>
	[SmallBasicType]
	public sealed class Shapes
	{
		private static Dictionary<string, int> _nameGenerationMap = new Dictionary<string, int>();

		private static Dictionary<string, RotateTransform> _rotateTransformMap = new Dictionary<string, RotateTransform>();

		private static Dictionary<string, ScaleTransform> _scaleTransformMap = new Dictionary<string, ScaleTransform>();

		private static Dictionary<string, Point> _positionMap = new Dictionary<string, Point>();

		/// <summary>
		/// Adds a rectangle shape with the specified width and height.
		/// </summary>
		/// <param name="width">
		/// The width of the rectangle shape.
		/// </param>
		/// <param name="height">
		/// The height of the rectangle shape.
		/// </param>
		/// <returns>
		/// The Rectangle shape that was just added to the Graphics Window.
		/// </returns>
		public static Primitive AddRectangle(Primitive width, Primitive height)
		{
			string name = GenerateNewName("Rectangle");
			GraphicsWindow.Invoke(delegate
			{
				GraphicsWindow.VerifyAccess();
				Rectangle shape = new Rectangle
				{
					Width = width,
					Height = height,
					Fill = GraphicsWindow._fillBrush,
					Stroke = GraphicsWindow._pen.Brush,
					StrokeThickness = GraphicsWindow._pen.Thickness
				};
				GraphicsWindow.AddShape(name, shape);
			});
			return name;
		}

		/// <summary>
		/// Adds an ellipse shape with the specified width and height.
		/// </summary>
		/// <param name="width">
		/// The width of the ellipse shape.
		/// </param>
		/// <param name="height">
		/// The height of the ellipse shape.
		/// </param>
		/// <returns>
		/// The Ellipse shape that was just added to the Graphics Window.
		/// </returns>
		public static Primitive AddEllipse(Primitive width, Primitive height)
		{
			string name = GenerateNewName("Ellipse");
			GraphicsWindow.Invoke(delegate
			{
				GraphicsWindow.VerifyAccess();
				Ellipse shape = new Ellipse
				{
					Width = width,
					Height = height,
					Fill = GraphicsWindow._fillBrush,
					Stroke = GraphicsWindow._pen.Brush,
					StrokeThickness = GraphicsWindow._pen.Thickness
				};
				GraphicsWindow.AddShape(name, shape);
			});
			return name;
		}

		/// <summary>
		/// Adds a triangle shape represented by the specified points.
		/// </summary>
		/// <param name="x1">
		/// The x co-ordinate of the first point.
		/// </param>
		/// <param name="y1">
		/// The y co-ordinate of the first point.
		/// </param>
		/// <param name="x2">
		/// The x co-ordinate of the second point.
		/// </param>
		/// <param name="y2">
		/// The y co-ordinate of the second point.
		/// </param>
		/// <param name="x3">
		/// The x co-ordinate of the third point.
		/// </param>
		/// <param name="y3">
		/// The y co-ordinate of the third point.
		/// </param>
		/// <returns>
		/// The Triangle shape that was just added to the Graphics Window.
		/// </returns>
		public static Primitive AddTriangle(Primitive x1, Primitive y1, Primitive x2, Primitive y2, Primitive x3, Primitive y3)
		{
			string name = GenerateNewName("Triangle");
			GraphicsWindow.Invoke(delegate
			{
				Polygon polygon = new Polygon();
				GraphicsWindow.AddShape(name, polygon);
				polygon.Points.Add(new Point(x1, y1));
				polygon.Points.Add(new Point(x2, y2));
				polygon.Points.Add(new Point(x3, y3));
				polygon.Fill = GraphicsWindow._fillBrush;
				polygon.Stroke = GraphicsWindow._pen.Brush;
				polygon.StrokeThickness = GraphicsWindow._pen.Thickness;
			});
			return name;
		}

		/// <summary>
		/// Adds a line between the specified points.
		/// </summary>
		/// <param name="x1">
		/// The x co-ordinate of the first point.
		/// </param>
		/// <param name="y1">
		/// The y co-ordinate of the first point.
		/// </param>
		/// <param name="x2">
		/// The x co-ordinate of the second point.
		/// </param>
		/// <param name="y2">
		/// The y co-ordinate of the second point.
		/// </param>
		/// <returns>
		/// The line that was just added to the Graphics Window.
		/// </returns>
		public static Primitive AddLine(Primitive x1, Primitive y1, Primitive x2, Primitive y2)
		{
			string name = GenerateNewName("Line");
			GraphicsWindow.Invoke(delegate
			{
				Line line = new Line();
				GraphicsWindow.AddShape(name, line);
				line.X1 = x1;
				line.Y1 = y1;
				line.X2 = x2;
				line.Y2 = y2;
				line.Stroke = GraphicsWindow._pen.Brush;
				line.StrokeThickness = GraphicsWindow._pen.Thickness;
			});
			return name;
		}

		/// <summary>
		/// Adds an image as a shape that can be moved, animated or rotated.
		/// </summary>
		/// <param name="imageName">
		/// The name of the image to draw.
		/// </param>
		/// <returns>
		/// The image that was just added to the Graphics Window.
		/// </returns>
		public static Primitive AddImage(Primitive imageName)
		{
			string name = GenerateNewName("Image");
			GraphicsWindow.Invoke(delegate
			{
				Image image = new Image();
				BitmapSource bitmap = ImageList.GetBitmap(imageName);
				if (bitmap != null)
				{
					image.Source = bitmap;
					if (bitmap.PixelWidth != 0 && bitmap.PixelHeight != 0)
					{
						image.Width = bitmap.PixelWidth;
						image.Height = bitmap.PixelHeight;
					}
					GraphicsWindow.AddShape(name, image);
				}
			});
			return name;
		}

		/// <summary>
		/// Adds some text as a shape that can be moved, animated or rotated.
		/// </summary>
		/// <param name="text">
		/// The text to add.
		/// </param>
		/// <returns>
		/// The text shape that was just added to the Graphics Window.
		/// </returns>
		public static Primitive AddText(Primitive text)
		{
			string name = GenerateNewName("Text");
			GraphicsWindow.Invoke(delegate
			{
				GraphicsWindow.VerifyAccess();
				TextBlock shape = new TextBlock
				{
					Text = text,
					Foreground = GraphicsWindow._fillBrush,
					FontFamily = GraphicsWindow._fontFamily,
					FontStyle = GraphicsWindow._fontStyle,
					FontSize = GraphicsWindow._fontSize,
					FontWeight = GraphicsWindow._fontWeight
				};
				GraphicsWindow.AddShape(name, shape);
			});
			return name;
		}

		/// <summary>
		/// Sets the text of a text shape. 
		/// </summary>
		/// <param name="shapeName">
		/// The name of the text shape.
		/// </param>
		/// <param name="text">
		/// The new text value to set.
		/// </param>
		public static void SetText(Primitive shapeName, Primitive text)
		{
			if (!GraphicsWindow._objectsMap.TryGetValue(shapeName, out var value))
			{
				return;
			}
			TextBlock textBlock = value as TextBlock;
			if (textBlock != null)
			{
				GraphicsWindow.BeginInvoke(delegate
				{
					textBlock.Text = text;
				});
			}
		}

		/// <summary>
		/// Removes a shape from the Graphics Window.
		/// </summary>
		/// <param name="shapeName">
		/// The name of the shape that needs to be removed.
		/// </param>
		public static void Remove(Primitive shapeName)
		{
			GraphicsWindow.RemoveShape(shapeName);
		}

		/// <summary>
		/// Moves the shape with the specified name to a new position.
		/// </summary>
		/// <param name="shapeName">
		/// The name of the shape to move.
		/// </param>
		/// <param name="x">
		/// The x co-ordinate of the new position.
		/// </param>
		/// <param name="y">
		/// The y co-ordinate of the new position.
		/// </param>
		public static void Move(Primitive shapeName, Primitive x, Primitive y)
		{
			if (GraphicsWindow._objectsMap.TryGetValue(shapeName, out var obj))
			{
				_positionMap[shapeName] = new Point(x, y);
				GraphicsWindow.BeginInvoke(delegate
				{
					obj.BeginAnimation(Canvas.LeftProperty, null);
					obj.BeginAnimation(Canvas.TopProperty, null);
					Canvas.SetLeft(obj, x);
					Canvas.SetTop(obj, y);
				});
			}
		}

		/// <summary>
		/// Rotates the shape with the specified name to the specified angle.
		/// </summary>
		/// <param name="shapeName">
		/// The name of the shape to rotate.
		/// </param>
		/// <param name="angle">
		/// The angle to rotate the shape.
		/// </param>
		public static void Rotate(Primitive shapeName, Primitive angle)
		{
			if (!GraphicsWindow._objectsMap.TryGetValue(shapeName, out var obj))
			{
				return;
			}
			GraphicsWindow.BeginInvoke(delegate
			{
				if (!(obj.RenderTransform is TransformGroup))
				{
					obj.RenderTransform = new TransformGroup();
				}
				if (!_rotateTransformMap.TryGetValue(shapeName, out var value))
				{
					value = new RotateTransform();
					_rotateTransformMap[shapeName] = value;
					FrameworkElement frameworkElement = obj as FrameworkElement;
					if (frameworkElement != null)
					{
						value.CenterX = frameworkElement.ActualWidth / 2.0;
						value.CenterY = frameworkElement.ActualHeight / 2.0;
					}
					((TransformGroup)obj.RenderTransform).Children.Add(value);
				}
				value.Angle = angle;
			});
		}

		/// <summary>
		/// Scales the shape using the specified zoom levels.  Minimum is 0.1 and maximum is 20.
		/// </summary>
		/// <param name="shapeName">
		/// The name of the shape to zoom.
		/// </param>
		/// <param name="scaleX">
		/// The x-axis zoom level.
		/// </param>
		/// <param name="scaleY">
		/// The y-axis zoom level.
		/// </param>
		public static void Zoom(Primitive shapeName, Primitive scaleX, Primitive scaleY)
		{
			if (!GraphicsWindow._objectsMap.TryGetValue(shapeName, out var obj))
			{
				return;
			}
			scaleX = Math.Min(Math.Max(scaleX, 0.1), 20.0);
			scaleY = Math.Min(Math.Max(scaleY, 0.1), 20.0);
			GraphicsWindow.BeginInvoke(delegate
			{
				if (!(obj.RenderTransform is TransformGroup))
				{
					obj.RenderTransform = new TransformGroup();
				}
				if (!_scaleTransformMap.TryGetValue(shapeName, out var value))
				{
					value = new ScaleTransform();
					_scaleTransformMap[shapeName] = value;
					FrameworkElement frameworkElement = obj as FrameworkElement;
					if (frameworkElement != null)
					{
						value.CenterX = frameworkElement.ActualWidth / 2.0;
						value.CenterY = frameworkElement.ActualHeight / 2.0;
					}
					((TransformGroup)obj.RenderTransform).Children.Add(value);
				}
				value.ScaleX = scaleX;
				value.ScaleY = scaleY;
			});
		}

		/// <summary>
		/// Animates a shape with the specified name to a new position.
		/// </summary>
		/// <param name="shapeName">
		/// The name of the shape to move.
		/// </param>
		/// <param name="x">
		/// The x co-ordinate of the new position.
		/// </param>
		/// <param name="y">
		/// The y co-ordinate of the new position.
		/// </param>
		/// <param name="duration">
		/// The time for the animation, in milliseconds.
		/// </param>
		public static void Animate(Primitive shapeName, Primitive x, Primitive y, Primitive duration)
		{
			if (GraphicsWindow._objectsMap.TryGetValue(shapeName, out var obj))
			{
				_positionMap[shapeName] = new Point(x, y);
				GraphicsWindow.Invoke(delegate
				{
					GraphicsWindow.DoubleAnimateProperty(obj, Canvas.LeftProperty, x, duration);
					GraphicsWindow.DoubleAnimateProperty(obj, Canvas.TopProperty, y, duration);
				});
			}
		}

		/// <summary>
		/// Gets the left co-ordinate of the specified shape.
		/// </summary>
		/// <param name="shapeName">
		/// The name of the shape.
		/// </param>
		/// <returns>
		/// The left co-ordinate of the shape.
		/// </returns>
		public static Primitive GetLeft(Primitive shapeName)
		{
			if (!GraphicsWindow._objectsMap.TryGetValue(shapeName, out var _))
			{
				return 0;
			}
			if (_positionMap.TryGetValue(shapeName, out var value2))
			{
				return value2.X;
			}
			return 0;
		}

		/// <summary>
		/// Gets the top co-ordinate of the specified shape.
		/// </summary>
		/// <param name="shapeName">
		/// The name of the shape.
		/// </param>
		/// <returns>
		/// The top co-ordinate of the shape.
		/// </returns>
		public static Primitive GetTop(Primitive shapeName)
		{
			if (!GraphicsWindow._objectsMap.TryGetValue(shapeName, out var _))
			{
				return 0;
			}
			if (_positionMap.TryGetValue(shapeName, out var value2))
			{
				return value2.Y;
			}
			return 0;
		}

		/// <summary>
		/// Gets the opacity of a shape.
		/// </summary>
		/// <param name="shapeName">
		/// The name of the shape.
		/// </param>
		/// <returns>
		/// The opacity of the object as a value between 0 and 100.  0 is completely transparent and 100 is completely opaque.
		/// </returns>
		public static Primitive GetOpacity(Primitive shapeName)
		{
			if (!GraphicsWindow._objectsMap.TryGetValue(shapeName, out var obj))
			{
				return 0;
			}
			return (Primitive)GraphicsWindow.InvokeWithReturn(() => (Primitive)obj.Opacity * (Primitive)100);
		}

		/// <summary>
		/// Sets how opaque a shape should render.
		/// </summary>
		/// <param name="shapeName">
		/// The name of the shape.
		/// </param>
		/// <param name="level">
		/// The opacity level ranging from 0 to 100.  0 is completely transparent and 100 is completely opaque.
		/// </param>
		public static void SetOpacity(Primitive shapeName, Primitive level)
		{
			if (GraphicsWindow._objectsMap.TryGetValue(shapeName, out var obj))
			{
				GraphicsWindow.Invoke(delegate
				{
					obj.Opacity = Math.Min(100, Math.Max(0, level)) / 100;
				});
			}
		}

		/// <summary>
		/// Hides an already added shape.
		/// </summary>
		/// <param name="shapeName">
		/// The name of the shape.
		/// </param>
		public static void HideShape(Primitive shapeName)
		{
			if (GraphicsWindow._objectsMap.TryGetValue(shapeName, out var obj))
			{
				GraphicsWindow.Invoke(delegate
				{
					obj.Visibility = Visibility.Collapsed;
				});
			}
		}

		/// <summary>
		/// Shows a previously hidden shape.
		/// </summary>
		/// <param name="shapeName">
		/// The name of the shape.
		/// </param>
		public static void ShowShape(Primitive shapeName)
		{
			if (GraphicsWindow._objectsMap.TryGetValue(shapeName, out var obj))
			{
				GraphicsWindow.Invoke(delegate
				{
					obj.Visibility = Visibility.Visible;
				});
			}
		}

		internal static string GenerateNewName(string prefix)
		{
			int value = 0;
			_nameGenerationMap.TryGetValue(prefix, out value);
			value++;
			_nameGenerationMap[prefix] = value;
			return prefix + value;
		}
	}
}
