using System;
using System.Collections.Generic;
using System.IO;
using System.Media;
using System.Reflection;
using System.Threading;
using System.Windows.Media;
using Microsoft.SmallBasic.Library.Internal;

namespace Microsoft.SmallBasic.Library
{
	/// <summary>
	/// The Sound object provides operations that allow the playback of sounds.  Some sample sounds are provided along with the library.
	/// </summary>
	[SmallBasicType]
	public sealed class Sound
	{
		private static Dictionary<Uri, MediaPlayer> _mediaPlayerMap = new Dictionary<Uri, MediaPlayer>();

		private static int _midiOut = 0;

		private static int _octave = 4;

		private static int _defaultLength = 4;

		private static Dictionary<string, int> _notes = new Dictionary<string, int>
		{
			{
				"C",
				0
			},
			{
				"C+",
				1
			},
			{
				"C#",
				1
			},
			{
				"D-",
				1
			},
			{
				"D",
				2
			},
			{
				"D+",
				3
			},
			{
				"D#",
				3
			},
			{
				"E-",
				3
			},
			{
				"E",
				4
			},
			{
				"F",
				5
			},
			{
				"F+",
				6
			},
			{
				"F#",
				6
			},
			{
				"G-",
				6
			},
			{
				"G",
				7
			},
			{
				"G+",
				8
			},
			{
				"G#",
				8
			},
			{
				"A-",
				8
			},
			{
				"A",
				9
			},
			{
				"A+",
				10
			},
			{
				"A#",
				10
			},
			{
				"B-",
				10
			},
			{
				"B",
				11
			}
		};

		/// <summary>
		/// Plays the Click Sound.
		/// </summary>
		public static void PlayClick()
		{
			PlayStockSound("Click.wav", sync: false);
		}

		/// <summary>
		/// Plays the Click Sound and waits for it to finish.
		/// </summary>
		public static void PlayClickAndWait()
		{
			PlayStockSound("Click.wav", sync: true);
		}

		/// <summary>
		/// Plays the Chime Sound.
		/// </summary>
		public static void PlayChime()
		{
			PlayStockSound("Chime.wav", sync: false);
		}

		/// <summary>
		/// Plays the Chime Sound and waits for it to finish.
		/// </summary>
		public static void PlayChimeAndWait()
		{
			PlayStockSound("Chime.wav", sync: true);
		}

		/// <summary>
		/// Plays the Chimes Sound.
		/// </summary>
		public static void PlayChimes()
		{
			PlayStockSound("Pause.wav", sync: false);
		}

		/// <summary>
		/// Plays the Chimes Sound and waits for it to finish.
		/// </summary>
		public static void PlayChimesAndWait()
		{
			PlayStockSound("Pause.wav", sync: true);
		}

		/// <summary>
		/// Plays the Bell Ring Sound.
		/// </summary>
		public static void PlayBellRing()
		{
			PlayStockSound("BellRing.wav", sync: false);
		}

		/// <summary>
		/// Plays the Bell Ring Sound and waits for it to finish.
		/// </summary>
		public static void PlayBellRingAndWait()
		{
			PlayStockSound("BellRing.wav", sync: true);
		}

		/// <summary>
		/// Plays musical notes.
		/// </summary>
		/// <param name="notes">
		/// A set of musical notes to play.  The format is a subset of the Music Macro Language supported by QBasic.
		/// </param>
		/// <example>
		/// <code>
		/// Sound.PlayMusic("O5 C8 C8 G8 G8 A8 A8 G4 F8 F8 E8 E8 D8 D8 C4")
		/// </code>
		/// </example>
		public static void PlayMusic(Primitive notes)
		{
			EnsureDeviceInit();
			PlayNotes(notes);
		}

		/// <summary>
		/// <para>
		/// Plays an audio file.  This could be an mp3 or wav or wma file.  Other file formats may or may not play depending on the audio codecs installed on the user's computer.
		/// </para>
		/// <para>
		/// If the file was already paused, this operation will resume from the position where the playback was paused.
		/// </para>
		/// </summary>
		/// <param name="filePath">
		/// The path for the audio file.  This could either be a local file (e.g.: c:\music\track1.mp3) or a file on the network (e.g.: http://contoso.com/track01.wma).
		/// </param>
		public static void Play(Primitive filePath)
		{
			GetMediaPlayer(filePath)?.Play();
		}

		/// <summary>
		/// <para>
		/// Plays an audio file and waits until it is finished playing.  This could be an mp3 or wav or wma file. Other file formats may or may not play depending on the audio codecs installed on the user's computer.
		/// </para>
		/// <para>
		/// If the file was already paused, this operation will resume from the position where the playback was paused.
		/// </para>
		/// </summary>
		/// <param name="filePath">
		/// The path for the audio file.  This could either be a local file (e.g.: c:\music\track1.mp3) or a file on the network (e.g.: http://contoso.com/track01.wma).
		/// </param>
		public static void PlayAndWait(Primitive filePath)
		{
			MediaPlayer mediaPlayer = GetMediaPlayer(filePath);
			if (mediaPlayer == null)
			{
				return;
			}
			AutoResetEvent autoResetEvent = new AutoResetEvent(initialState: false);
			mediaPlayer.Play();
			int num = 0;
			while (!autoResetEvent.WaitOne(200, exitContext: false))
			{
				if (num > 30 && mediaPlayer.Position == TimeSpan.Zero)
				{
					autoResetEvent.Set();
				}
				if (mediaPlayer.NaturalDuration.HasTimeSpan && mediaPlayer.Position >= mediaPlayer.NaturalDuration.TimeSpan)
				{
					autoResetEvent.Set();
				}
				num++;
			}
			mediaPlayer.Stop();
		}

		/// <summary>
		/// Pauses playback of an audio file.  If the file was not already playing, this operation will not do anything.
		/// </summary>
		/// <param name="filePath">
		/// The path for the audio file.  This could either be a local file (e.g.: c:\music\track1.mp3) or a file on the network (e.g.: http://contoso.com/track01.wma).
		/// </param>
		public static void Pause(Primitive filePath)
		{
			GetMediaPlayer(filePath)?.Pause();
		}

		/// <summary>
		/// Stops playback of an audio file.  If the file was not already playing, this operation will not do anything.
		/// </summary>
		/// <param name="filePath">
		/// The path for the audio file.  This could either be a local file (e.g.: c:\music\track1.mp3) or a file on the network (e.g.: http://contoso.com/track01.wma).
		/// </param>
		public static void Stop(Primitive filePath)
		{
			GetMediaPlayer(filePath)?.Stop();
		}

		private static void PlayStockSound(string name, bool sync)
		{
			SoundPlayer soundPlayer = new SoundPlayer(GetSoundStream(name));
			if (sync)
			{
				soundPlayer.PlaySync();
			}
			else
			{
				soundPlayer.Play();
			}
		}

		private static Stream GetSoundStream(string fileName)
		{
			Assembly executingAssembly = Assembly.GetExecutingAssembly();
			string name = $"Sounds.{fileName}";
			return executingAssembly.GetManifestResourceStream(typeof(Sound), name);
		}

		private static MediaPlayer GetMediaPlayer(Primitive filePath)
		{
			if (filePath.IsEmpty)
			{
				return null;
			}
			try
			{
				Uri uri = new Uri(filePath);
				if (!_mediaPlayerMap.TryGetValue(uri, out var value))
				{
					value = new MediaPlayer();
					_mediaPlayerMap[uri] = value;
					value.Open(uri);
				}
				return value;
			}
			catch (Exception)
			{
			}
			return null;
		}

		private static void EnsureDeviceInit()
		{
			if (_midiOut == 0)
			{
				NativeHelper.midiOutOpen(ref _midiOut, 0, 0, 0, 0u);
				ChangeInstrument(0);
			}
		}

		private static void PlayNotes(string song)
		{
			int num = 0;
			song = song.ToUpperInvariant();
			int length = song.Length;
			while (num < song.Length)
			{
				int num2 = _defaultLength;
				char c = song[num++];
				if (char.IsLetter(c))
				{
					string text = new string(c, 1);
					if (num < length)
					{
						c = song[num];
						if (c == '#' || c == '+' || c == '-')
						{
							text += new string(c, 1);
							num++;
						}
						if (num < length)
						{
							c = song[num];
							if (char.IsDigit(c))
							{
								num2 = c - 48;
								num++;
							}
							if (num < length)
							{
								c = song[num];
								if (char.IsDigit(c))
								{
									num2 = num2 * 10 + (c - 48);
									num++;
								}
							}
						}
					}
					if (text[0] == 'O')
					{
						_octave = num2;
					}
					else
					{
						PlayNote(_octave, text, num2);
					}
				}
				else
				{
					switch (c)
					{
					case '>':
						_octave = Math.Min(8, _octave + 1);
						break;
					case '<':
						_octave = Math.Max(0, _octave - 1);
						break;
					}
				}
			}
		}

		private static void PlayNote(int octave, string note, int length)
		{
			double num = 1600.0 / (double)length;
			switch (note)
			{
			case "P":
			case "R":
				Thread.Sleep((int)num);
				return;
			case "L":
				_defaultLength = length;
				return;
			}
			if (!_notes.TryGetValue(note, out var value))
			{
				value = 0;
			}
			octave = Math.Min(Math.Max(0, octave), 8);
			int number = octave * 12 + value;
			PlayNote(number);
			Thread.Sleep((int)num);
		}

		private static void PlayNote(int number)
		{
			uint dwMsg = BitConverter.ToUInt32(new byte[4]
			{
				144,
				(byte)number,
				100,
				0
			}, 0);
			NativeHelper.midiOutShortMsg(_midiOut, dwMsg);
		}

		private static void ChangeInstrument(int number)
		{
			uint dwMsg = BitConverter.ToUInt32(new byte[4]
			{
				192,
				(byte)number,
				0,
				0
			}, 0);
			NativeHelper.midiOutShortMsg(_midiOut, dwMsg);
		}
	}
}
