using System;
using System.Text;
using Microsoft.SmallBasic.Library.Internal;

namespace Microsoft.SmallBasic.Library
{
	/// <summary>
	/// The TextWindow provides text-related input and output functionalities.  For example using this class, it is possible to write or read some text or number to and from the text-based text window.
	/// </summary>
	[SmallBasicType]
	public sealed class TextWindow
	{
		private static bool _windowVisible;

		/// <summary>
		/// Gets or sets the foreground color of the text to be output in the text window.
		/// </summary>
		public static Primitive ForegroundColor
		{
			get
			{
				VerifyAccess();
				return Console.ForegroundColor.ToString();
			}
			set
			{
				VerifyAccess();
				try
				{
					Console.ForegroundColor = (ConsoleColor)Enum.Parse(typeof(ConsoleColor), value, ignoreCase: true);
				}
				catch
				{
				}
			}
		}

		/// <summary>
		/// Gets or sets the background color of the text to be output in the text window.
		/// </summary>
		public static Primitive BackgroundColor
		{
			get
			{
				VerifyAccess();
				return Console.BackgroundColor.ToString();
			}
			set
			{
				VerifyAccess();
				try
				{
					Console.BackgroundColor = (ConsoleColor)Enum.Parse(typeof(ConsoleColor), value, ignoreCase: true);
				}
				catch
				{
				}
			}
		}

		/// <summary>
		/// Gets or sets the cursor's column position on the text window.
		/// </summary>
		public static Primitive CursorLeft
		{
			get
			{
				VerifyAccess();
				return new Primitive(Console.CursorLeft);
			}
			set
			{
				VerifyAccess();
				Console.CursorLeft = value;
			}
		}

		/// <summary>
		/// Gets or sets the cursor's row position on the text window.
		/// </summary>
		public static Primitive CursorTop
		{
			get
			{
				VerifyAccess();
				return new Primitive(Console.CursorTop);
			}
			set
			{
				VerifyAccess();
				Console.CursorTop = value;
			}
		}

		/// <summary>
		/// Gets or sets the Left position of the Text Window.
		/// </summary>
		public static Primitive Left
		{
			get
			{
				VerifyAccess();
				IntPtr consoleWindow = NativeHelper.GetConsoleWindow();
				NativeHelper.GetWindowRect(consoleWindow, out var lpRect);
				return lpRect.Left;
			}
			set
			{
				VerifyAccess();
				IntPtr consoleWindow = NativeHelper.GetConsoleWindow();
				NativeHelper.GetWindowRect(consoleWindow, out var lpRect);
				NativeHelper.SetWindowPos(consoleWindow, IntPtr.Zero, value, lpRect.Top, 0, 0, 1u);
			}
		}

		/// <summary>
		/// Gets or sets the Title for the text window.
		/// </summary>
		public static Primitive Title
		{
			get
			{
				VerifyAccess();
				return new Primitive(Console.Title);
			}
			set
			{
				VerifyAccess();
				Console.Title = value;
			}
		}

		/// <summary>
		/// Gets or sets the Top position of the Text Window.
		/// </summary>
		public static Primitive Top
		{
			get
			{
				VerifyAccess();
				IntPtr consoleWindow = NativeHelper.GetConsoleWindow();
				NativeHelper.GetWindowRect(consoleWindow, out var lpRect);
				return lpRect.Top;
			}
			set
			{
				VerifyAccess();
				IntPtr consoleWindow = NativeHelper.GetConsoleWindow();
				NativeHelper.GetWindowRect(consoleWindow, out var lpRect);
				NativeHelper.SetWindowPos(consoleWindow, IntPtr.Zero, lpRect.Left, value, 0, 0, 1u);
			}
		}

		/// <summary>
		/// Shows the Text window to enable interactions with it.
		/// </summary>
		public static void Show()
		{
			if (!_windowVisible)
			{
				IntPtr consoleWindow = NativeHelper.GetConsoleWindow();
				if (consoleWindow == IntPtr.Zero)
				{
					NativeHelper.AllocConsole();
				}
				NativeHelper.ShowWindow(consoleWindow, 8);
				_windowVisible = true;
			}
		}

		/// <summary>
		/// Hides the text window.  Content is perserved when the window is shown again.
		/// </summary>
		public static void Hide()
		{
			if (_windowVisible)
			{
				IntPtr consoleWindow = NativeHelper.GetConsoleWindow();
				if (consoleWindow != IntPtr.Zero)
				{
					NativeHelper.ShowWindow(consoleWindow, 0);
				}
				_windowVisible = false;
			}
		}

		/// <summary>
		/// Clears the TextWindow.
		/// </summary>
		public static void Clear()
		{
			VerifyAccess();
			Console.Clear();
		}

		/// <summary>
		/// Waits for user input before returning.
		/// </summary>
		public static void Pause()
		{
			VerifyAccess();
			Console.WriteLine("Press any key to continue...");
			Console.ReadKey(intercept: true);
		}

		/// <summary>
		/// Waits for user input only when the TextWindow is already open.
		/// </summary>
		public static void PauseIfVisible()
		{
			if (_windowVisible)
			{
				Pause();
			}
		}

		/// <summary>
		/// Waits for user input before returning.
		/// </summary>
		public static void PauseWithoutMessage()
		{
			VerifyAccess();
			Console.ReadKey(intercept: true);
		}

		/// <summary>
		/// Reads a line of text from the text window.  This function will not return until the user hits ENTER.
		/// </summary>
		/// <returns>
		/// The text that was read from the text window
		/// </returns>
		public static Primitive Read()
		{
			VerifyAccess();
			return new Primitive(Console.ReadLine());
		}

		/// <summary>
		/// Reads a single character from the text window.  
		/// </summary>
		/// <returns>
		/// The character that was read from the text window.
		/// </returns>
		[HideFromIntellisense]
		public static Primitive ReadKey()
		{
			VerifyAccess();
			return new string(Console.ReadKey(intercept: true).KeyChar, 1);
		}

		/// <summary>
		/// Reads a number from the text window.  This function will not return until the user hits ENTER.
		/// </summary>
		/// <returns>
		/// The number that was read from the text window
		/// </returns>
		public static Primitive ReadNumber()
		{
			VerifyAccess();
			StringBuilder stringBuilder = new StringBuilder();
			bool flag = false;
			int num = 0;
			while (true)
			{
				ConsoleKeyInfo consoleKeyInfo = Console.ReadKey(intercept: true);
				char keyChar = consoleKeyInfo.KeyChar;
				bool flag2 = false;
				if (keyChar == '-' && num == 0)
				{
					flag2 = true;
				}
				else if (keyChar == '.' && !flag)
				{
					flag = true;
					flag2 = true;
				}
				else if (keyChar >= '0' && keyChar <= '9')
				{
					flag2 = true;
				}
				if (flag2)
				{
					Console.Write(keyChar);
					stringBuilder.Append(keyChar);
					num++;
				}
				else if (num > 0 && consoleKeyInfo.Key == ConsoleKey.Backspace)
				{
					Console.CursorLeft--;
					Console.Write(" ");
					Console.CursorLeft--;
					num--;
					keyChar = stringBuilder[num];
					if (keyChar == '.')
					{
						flag = false;
					}
					stringBuilder.Remove(num, 1);
				}
				else if (consoleKeyInfo.Key == ConsoleKey.Enter)
				{
					break;
				}
			}
			Console.WriteLine();
			if (stringBuilder.Length == 0)
			{
				return new Primitive(0);
			}
			return new Primitive(stringBuilder.ToString());
		}

		/// <summary>
		/// Writes text or number to the text window.  A new line character will be appended to the output, so that the next time something is written to the text window, it will go in a new line.
		/// </summary>
		/// <param name="data">
		/// The text or number to write to the text window.
		/// </param>
		public static void WriteLine(Primitive data)
		{
			VerifyAccess();
			Console.WriteLine((string)data);
		}

		/// <summary>
		/// Writes text or number to the text window.  Unlike WriteLine, this will not append a new line character, which means, anything written to the text window after this call will be on the same line.
		/// </summary>
		/// <param name="data">
		/// The text or number to write to the text window
		/// </param>
		public static void Write(Primitive data)
		{
			VerifyAccess();
			string x = data;
			Console.Write(x);
		}

		/// <summary>
		/// Verifies if the access to text Window has been made yet
		/// </summary>
		private static void VerifyAccess()
		{
			if (!_windowVisible)
			{
				Show();
			}
		}
	}
}
