using System.Threading;

namespace Microsoft.SmallBasic.Library
{
	/// <summary>
	/// The Timer object provides an easy way for doing something repeatedly with a constant interval between.
	/// </summary>
	[SmallBasicType]
	public sealed class Timer
	{
		public const int _maxInterval = 100000000;

		private static int _interval;

		private static System.Threading.Timer _threadTimer;

		/// <summary>
		/// Gets or sets the interval (in milliseconds) specifying how often the timer should raise the Tick event.  This value can range from 10 to 100000000
		/// </summary>
		public static Primitive Interval
		{
			get
			{
				return _interval;
			}
			set
			{
				_interval = value;
				_interval = Math.Max(10, Math.Min(_interval, 100000000));
				_threadTimer.Change(_interval, _interval);
			}
		}

		private static event SmallBasicCallback _tick;

		/// <summary>
		/// Raises an event when the timer ticks.
		/// </summary>
		public static event SmallBasicCallback Tick
		{
			add
			{
				Timer._tick = null;
				_tick += value;
			}
			remove
			{
				_tick -= value;
			}
		}

		static Timer()
		{
			_interval = 100000000;
			_threadTimer = new System.Threading.Timer(ThreadTimerCallback);
		}

		/// <summary>
		/// Pauses the timer.  Tick events will not be raised.
		/// </summary>
		public static void Pause()
		{
			_threadTimer.Change(-1, -1);
		}

		/// <summary>
		/// Resumes the timer from a paused state.  Tick events will now be raised.
		/// </summary>
		public static void Resume()
		{
			_threadTimer.Change(_interval, _interval);
		}

		private static void ThreadTimerCallback(object tag)
		{
			if (Timer._tick != null)
			{
				Timer._tick();
			}
		}
	}
}
